@@ -57,11 +57,11 @@ SIGNING_SCOPE_ALIAS_TO_PROJECT = [[
     'all-release-branches', set([
         'mozilla-beta',
         'mozilla-release',
-        'mozilla-esr60',
         'mozilla-esr68',
+        'mozilla-esr78',
         'comm-beta',
-        'comm-esr60',
         'comm-esr68',
+        'comm-esr78',
     ])
 ]]
 
@@ -73,27 +73,6 @@ SIGNING_CERT_SCOPES = {
     'default': 'signing:cert:dep-signing',
 }
 
-ANDROID_SIGNING_SCOPE_ALIAS_TO_PROJECT = [[
-    '68-release-train', set([
-        'mozilla-beta',
-        'mozilla-release',
-        'mozilla-esr68',
-    ])
-]]
-
-ANDROID_SIGNING_CERT_SCOPES = {
-    '68-release-train': {
-        'nightly': 'signing:cert:nightly-signing',
-        'beta': 'signing:cert:release-signing',
-        'release': 'signing:cert:release-signing',
-    },
-    'default': {
-        'nightly': 'signing:cert:dep-signing',
-        'beta': 'signing:cert:dep-signing',
-        'release': 'signing:cert:dep-signing',
-    }
-}
-
 DEVEDITION_SIGNING_SCOPE_ALIAS_TO_PROJECT = [[
     'beta', set([
         'mozilla-beta',
@@ -117,11 +96,11 @@ BEETMOVER_SCOPE_ALIAS_TO_PROJECT = [[
     'all-release-branches', set([
         'mozilla-beta',
         'mozilla-release',
-        'mozilla-esr60',
         'mozilla-esr68',
+        'mozilla-esr78',
         'comm-beta',
-        'comm-esr60',
         'comm-esr68',
+        'comm-esr78',
     ])
 ]]
 
@@ -133,27 +112,6 @@ BEETMOVER_BUCKET_SCOPES = {
     'default': 'beetmover:bucket:dep',
 }
 
-ANDROID_BEETMOVER_SCOPE_ALIAS_TO_PROJECT = [[
-    '68-release-train', set([
-        'mozilla-beta',
-        'mozilla-release',
-        'mozilla-esr68',
-    ])
-]]
-
-ANDROID_BEETMOVER_BUCKET_SCOPES = {
-    '68-release-train': {
-        'nightly': 'beetmover:bucket:nightly',
-        'beta': 'beetmover:bucket:release',
-        'release': 'beetmover:bucket:release',
-    },
-    'default': {
-        'nightly': 'beetmover:bucket:dep',
-        'beta': 'beetmover:bucket:dep',
-        'release': 'beetmover:bucket:dep',
-    },
-}
-
 """Map the beetmover tasks aliases to the actual action scopes.
 """
 BEETMOVER_ACTION_SCOPES = {
@@ -162,13 +120,6 @@ BEETMOVER_ACTION_SCOPES = {
     'default': 'beetmover:action:push-to-candidates',
 }
 
-ANDROID_BEETMOVER_ACTION_SCOPES = {
-    'default': {
-        'nightly': 'beetmover:action:push-to-nightly',
-        'beta': 'beetmover:action:push-to-candidates',
-        'release': 'beetmover:action:push-to-candidates',
-    },
-}
 
 """Known balrog actions."""
 BALROG_ACTIONS = ('submit-locale', 'submit-toplevel', 'schedule')
@@ -191,17 +142,17 @@ BALROG_SCOPE_ALIAS_TO_PROJECT = [[
 ], [
     'release', set([
         'mozilla-release',
-        'comm-esr60',
         'comm-esr68',
-    ])
-], [
-    'esr60', set([
-        'mozilla-esr60',
+        'comm-esr78',
     ])
 ], [
     'esr68', set([
         'mozilla-esr68',
     ])
+], [
+    'esr78', set([
+        'mozilla-esr78',
+    ])
 ]]
 
 """Map the balrog scope aliases to the actual scopes.
@@ -211,35 +162,12 @@ BALROG_SERVER_SCOPES = {
     'aurora': 'balrog:server:aurora',
     'beta': 'balrog:server:beta',
     'release': 'balrog:server:release',
-    'esr60': 'balrog:server:esr',
     'esr68': 'balrog:server:esr',
+    'esr78': 'balrog:server:esr',
     'default': 'balrog:server:dep',
 }
 
 
-PUSH_APK_SCOPE_ALIAS_TO_PROJECT = [[
-    '68-release-train', set([
-        'mozilla-beta',
-        'mozilla-release',
-        'mozilla-esr68',
-    ])
-]]
-
-
-PUSH_APK_SCOPES = {
-    '68-release-train': {
-        'nightly': 'googleplay:aurora',
-        'beta': 'googleplay:beta',
-        'release': 'googleplay:release',
-    },
-    'default': {
-        'nightly': 'googleplay:dep',
-        'beta': 'googleplay:dep',
-        'release': 'googleplay:dep',
-    }
-}
-
-
 """ The list of the release promotion phases which we send notifications for
 """
 RELEASE_NOTIFICATION_PHASES = ('promote', 'push', 'ship')
@@ -276,8 +204,8 @@ def with_scope_prefix(f):
         callable: the wrapped function
     """
     @functools.wraps(f)
-    def wrapper(config, *args, **kwargs):
-        scope_or_scopes = f(config, *args, **kwargs)
+    def wrapper(config, **kwargs):
+        scope_or_scopes = f(config, **kwargs)
         if isinstance(scope_or_scopes, list):
             return map(functools.partial(add_scope_prefix, config), scope_or_scopes)
         else:
@@ -306,16 +234,6 @@ def get_scope_from_project(config, alias_to_project_map, alias_to_scope_map):
     return alias_to_scope_map['default']
 
 
-@with_scope_prefix
-def get_scope_from_project_and_job_release_type(
-    config, job_release_type, alias_to_project_map, alias_to_scope_map
-):
-    for alias, projects in alias_to_project_map:
-        if config.params['project'] in projects and alias in alias_to_scope_map:
-            return alias_to_scope_map[alias][job_release_type]
-    return alias_to_scope_map['default'][job_release_type]
-
-
 @with_scope_prefix
 def get_scope_from_release_type(config, release_type_to_scope_map):
     """Determine the restricted scope from `config.params['target_tasks_method']`.
@@ -363,53 +281,29 @@ get_signing_cert_scope = functools.partial(
     alias_to_scope_map=SIGNING_CERT_SCOPES,
 )
 
-get_android_signing_cert_scope = functools.partial(
-    get_scope_from_project_and_job_release_type,
-    alias_to_project_map=ANDROID_SIGNING_SCOPE_ALIAS_TO_PROJECT,
-    alias_to_scope_map=ANDROID_SIGNING_CERT_SCOPES,
-)
-
 get_devedition_signing_cert_scope = functools.partial(
     get_scope_from_project,
     alias_to_project_map=DEVEDITION_SIGNING_SCOPE_ALIAS_TO_PROJECT,
     alias_to_scope_map=DEVEDITION_SIGNING_CERT_SCOPES,
 )
 
-get_beetmover_regular_bucket_scope = functools.partial(
+get_beetmover_bucket_scope = functools.partial(
     get_scope_from_project,
     alias_to_project_map=BEETMOVER_SCOPE_ALIAS_TO_PROJECT,
     alias_to_scope_map=BEETMOVER_BUCKET_SCOPES,
 )
 
-get_beetmover_regular_action_scope = functools.partial(
+get_beetmover_action_scope = functools.partial(
     get_scope_from_release_type,
     release_type_to_scope_map=BEETMOVER_ACTION_SCOPES,
 )
 
-get_beetmover_android_bucket_scope = functools.partial(
-    get_scope_from_project_and_job_release_type,
-    alias_to_project_map=ANDROID_BEETMOVER_SCOPE_ALIAS_TO_PROJECT,
-    alias_to_scope_map=ANDROID_BEETMOVER_BUCKET_SCOPES,
-)
-
-get_beetmover_android_action_scope = functools.partial(
-    get_scope_from_project_and_job_release_type,
-    alias_to_project_map=ANDROID_BEETMOVER_SCOPE_ALIAS_TO_PROJECT,
-    alias_to_scope_map=ANDROID_BEETMOVER_ACTION_SCOPES,
-)
-
 get_balrog_server_scope = functools.partial(
     get_scope_from_project,
     alias_to_project_map=BALROG_SCOPE_ALIAS_TO_PROJECT,
     alias_to_scope_map=BALROG_SERVER_SCOPES,
 )
 
-get_push_apk_scope = functools.partial(
-    get_scope_from_project_and_job_release_type,
-    alias_to_project_map=PUSH_APK_SCOPE_ALIAS_TO_PROJECT,
-    alias_to_scope_map=PUSH_APK_SCOPES,
-)
-
 cached_load_yaml = memoize(load_yaml)
 
 
@@ -444,66 +338,27 @@ def get_release_config(config):
         if release_config['partial_versions'] == "{}":
             del release_config['partial_versions']
 
-    release_config['version'] = str(config.params['version'])
-    release_config['appVersion'] = str(config.params['app_version'])
+    release_config['version'] = config.params['version']
+    release_config['appVersion'] = config.params['app_version']
 
-    release_config['next_version'] = str(config.params['next_version'])
+    release_config['next_version'] = config.params['next_version']
     release_config['build_number'] = config.params['build_number']
     return release_config
 
 
-def get_signing_cert_scope_per_platform(build_platform, is_nightly, config, job_release_type=None):
-    if 'android' in build_platform and job_release_type is not None:
-        return get_android_signing_cert_scope(config, job_release_type)
+def get_signing_cert_scope_per_platform(build_platform, is_shippable, config):
     if 'devedition' in build_platform:
         return get_devedition_signing_cert_scope(config)
-    elif is_nightly or build_platform in ('firefox-source', 'fennec-source', 'thunderbird-source'):
+    elif is_shippable:
         return get_signing_cert_scope(config)
     else:
         return add_scope_prefix(config, 'signing:cert:dep-signing')
 
 
-def get_beetmover_bucket_scope(config, job_release_type=None):
-    if job_release_type:
-        return get_beetmover_android_bucket_scope(config, job_release_type)
-    else:
-        return get_beetmover_regular_bucket_scope(config)
-
-
-def get_beetmover_action_scope(config, job_release_type=None):
-    if job_release_type:
-        return get_beetmover_android_action_scope(config, job_release_type)
-    else:
-        return get_beetmover_regular_action_scope(config)
-
-
-def get_worker_type_for_scope(config, scope):
-    """Get the scriptworker type that will accept the given scope.
-
-    Args:
-        config (TransformConfig): The configuration for the kind being transformed.
-        scope (string): The scope being used.
-
-    Returns:
-        string: The worker-type to use.
-    """
-    for worker_type, scopes in config.graph_config['scriptworker']['worker-types'].items():
-        if scope in scopes:
-            return worker_type
-    raise RuntimeError(
-        "Unsupported scriptworker scope {scope}. (supported scopes: {available_scopes})".format(
-            scope=scope,
-            available_scopes=sorted(
-                scope
-                for scopes in config.graph_config['scriptworker']['worker-types'].values()
-                for scope in scopes
-            ),
-        )
-    )
-
-
 # generate_beetmover_upstream_artifacts {{{1
-def generate_beetmover_upstream_artifacts(config, job, platform, locale=None, dependencies=None):
+def generate_beetmover_upstream_artifacts(
+    config, job, platform, locale=None, dependencies=None, **kwargs
+):
     """Generate the upstream artifacts for beetmover, using the artifact map.
 
     Currently only applies to beetmover tasks.
@@ -537,7 +392,12 @@ def generate_beetmover_upstream_artifacts(config, job, platform, locale=None, de
         locales = [locale]
 
     if not dependencies:
-        dependencies = job['dependencies'].keys()
+        if job.get('dependencies'):
+            dependencies = job['dependencies'].keys()
+        elif job.get('primary-dependency'):
+            dependencies = [job['primary-dependency'].kind]
+        else:
+            raise Exception('Unsupported type of dependency. Got job: {}'.format(job))
 
     for locale, dep in itertools.product(locales, dependencies):
         paths = list()
@@ -559,16 +419,23 @@ def generate_beetmover_upstream_artifacts(config, job, platform, locale=None, de
             file_config = deepcopy(map_config['mapping'][filename])
             resolve_keyed_by(file_config, "source_path_modifier",
                              'source path modifier', locale=locale)
+
+            kwargs['locale'] = locale
+
             paths.append(os.path.join(
                 base_artifact_prefix,
-                jsone.render(file_config['source_path_modifier'], {'locale': locale}),
-                filename,
+                jsone.render(file_config['source_path_modifier'], kwargs),
+                jsone.render(filename, kwargs),
             ))
 
-        if getattr(job['dependencies'][dep], 'release_artifacts', None):
+        if (
+            job.get('dependencies') and
+            getattr(job['dependencies'][dep], 'release_artifacts', None)
+        ):
             paths = [
                 path for path in paths
-                if path in job['dependencies'][dep].release_artifacts]
+                if path in job['dependencies'][dep].release_artifacts
+            ]
 
         if not paths:
             continue
@@ -585,57 +452,6 @@ def generate_beetmover_upstream_artifacts(config, job, platform, locale=None, de
     return upstream_artifacts
 
 
-# generate_beetmover_compressed_upstream_artifacts {{{1
-def generate_beetmover_compressed_upstream_artifacts(job, dependencies=None):
-    """Generate compressed file upstream artifacts for beetmover.
-
-    These artifacts will not be beetmoved directly, but will be
-    decompressed from upstream_mapping and the contents beetmoved
-    using the `mapping` entry in the artifact map.
-
-    Currently only applies to beetmover tasks.
-
-    Args:
-        job (dict): The current job being generated
-        dependencies (list): A list of the job's dependency labels.
-
-    Returns:
-        list: A list of dictionaries conforming to the upstream_artifacts spec.
-    """
-    base_artifact_prefix = get_artifact_prefix(job)
-    map_config = deepcopy(cached_load_yaml(job['attributes']['artifact_map']))
-    upstream_artifacts = list()
-
-    if not dependencies:
-        dependencies = job['dependencies'].keys()
-
-    for dep in dependencies:
-        paths = list()
-
-        for filename in map_config['upstream_mapping']:
-            if dep not in map_config['upstream_mapping'][filename]['from']:
-                continue
-
-            paths.append(os.path.join(
-                base_artifact_prefix,
-                filename,
-            ))
-
-        if not paths:
-            continue
-
-        upstream_artifacts.append({
-            "taskId": {
-                "task-reference": "<{}>".format(dep)
-            },
-            "taskType": map_config['tasktype_map'].get(dep),
-            "paths": sorted(paths),
-            "zipExtract": True,
-        })
-
-    return upstream_artifacts
-
-
 # generate_beetmover_artifact_map {{{1
 def generate_beetmover_artifact_map(config, job, **kwargs):
     """Generate the beetmover artifact map.
@@ -656,7 +472,7 @@ def generate_beetmover_artifact_map(config, job, **kwargs):
     platform = kwargs.get('platform', '')
     resolve_keyed_by(
         job, 'attributes.artifact_map',
-        'artifact map',
+        job['label'],
         **{
             'release-type': config.params['release_type'],
             'platform': platform,
@@ -677,7 +493,7 @@ def generate_beetmover_artifact_map(config, job, **kwargs):
     else:
         locales = map_config['default_locales']
 
-    resolve_keyed_by(map_config, 's3_bucket_paths', 's3_bucket_paths', platform=platform)
+    resolve_keyed_by(map_config, 's3_bucket_paths', job['label'], platform=platform)
 
     for locale, dep in itertools.product(locales, dependencies):
         paths = dict()
@@ -711,7 +527,9 @@ def generate_beetmover_artifact_map(config, job, **kwargs):
                 'pretty_name',
                 'checksums_path'
             ]:
-                resolve_keyed_by(file_config, field, field, locale=locale, platform=platform)
+                resolve_keyed_by(
+                    file_config, field, job["label"], locale=locale, platform=platform
+                )
 
             # This format string should ideally be in the configuration file,
             # but this would mean keeping variable names in sync between code + config.
@@ -754,7 +572,7 @@ def generate_beetmover_artifact_map(config, job, **kwargs):
         platforms = deepcopy(map_config.get('platform_names', {}))
         if platform:
             for key in platforms.keys():
-                resolve_keyed_by(platforms, key, key, platform=platform)
+                resolve_keyed_by(platforms, key, job['label'], platform=platform)
 
         upload_date = datetime.fromtimestamp(config.params['build_date'])
 
@@ -914,12 +732,3 @@ def generate_beetmover_partials_artifact_map(config, job, partials_info, **kwarg
         })
 
     return artifacts
-
-
-def should_use_artifact_map(platform):
-    """Return True if this task uses the beetmover artifact map.
-
-    This function exists solely for the beetmover artifact map
-    migration.
-    """
-    return 'devedition' not in platform
