@@ -21,7 +21,6 @@
 #include "nsGZFileWriter.h"
 #include "nsJSEnvironment.h"
 #include "nsPrintfCString.h"
-#include "nsISimpleEnumerator.h"
 #include "nsServiceManagerUtils.h"
 #include "nsIFile.h"
 
@@ -110,7 +109,7 @@ class GCAndCCLogDumpRunnable final : public Runnable,
   NS_IMETHOD OnFinish() override { return NS_OK; }
 
  private:
-  ~GCAndCCLogDumpRunnable() {}
+  ~GCAndCCLogDumpRunnable() = default;
 
   const nsString mIdentifier;
   const bool mDumpAllTraces;
@@ -241,9 +240,9 @@ void OnFifoEnabledChange(const char* /*unused*/, void* /*unused*/) {
 
 NS_IMPL_ISUPPORTS(nsMemoryInfoDumper, nsIMemoryInfoDumper)
 
-nsMemoryInfoDumper::nsMemoryInfoDumper() {}
+nsMemoryInfoDumper::nsMemoryInfoDumper() = default;
 
-nsMemoryInfoDumper::~nsMemoryInfoDumper() {}
+nsMemoryInfoDumper::~nsMemoryInfoDumper() = default;
 
 /* static */
 void nsMemoryInfoDumper::Initialize() {
@@ -397,6 +396,12 @@ class GZWriterWrapper : public JSONWriteFunc {
     Unused << mGZWriter->Write(aStr);
   }
 
+  void Write(const char* aStr, size_t aLen) override {
+    // Ignore any failure because JSONWriteFunc doesn't have a mechanism for
+    // handling errors.
+    Unused << mGZWriter->Write(aStr, aLen);
+  }
+
   nsresult Finish() { return mGZWriter->Finish(); }
 
  private:
@@ -431,6 +436,9 @@ class HandleReportAndFinishReportingCallbacks final
       // of the form "$PROCESS_NAME (pid $PID)", or just "(pid $PID)" if we
       // don't have a process name.  If we're the main process, we let
       // $PROCESS_NAME be "Main Process".
+      //
+      // `appendAboutMemoryMain()` in aboutMemory.js does much the same thing
+      // for live memory reports.
       if (XRE_IsParentProcess()) {
         // We're the main process.
         process.AssignLiteral("Main Process");
@@ -482,7 +490,7 @@ class HandleReportAndFinishReportingCallbacks final
   }
 
  private:
-  ~HandleReportAndFinishReportingCallbacks() {}
+  ~HandleReportAndFinishReportingCallbacks() = default;
 
   UniquePtr<JSONWriter> mWriter;
   nsCOMPtr<nsIFinishDumpingCallback> mFinishDumping;
@@ -560,7 +568,7 @@ class TempDirFinishCallback final : public nsIFinishDumpingCallback {
   }
 
  private:
-  ~TempDirFinishCallback() {}
+  ~TempDirFinishCallback() = default;
 
   nsCOMPtr<nsIFile> mReportsTmpFile;
   nsCString mReportsFilename;
