@@ -5,7 +5,6 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsAtomTable.h"
-#include "nsAutoPtr.h"
 #include "nsCOMPtr.h"
 #include "nsCOMArray.h"
 #include "nsPrintfCString.h"
@@ -69,7 +68,7 @@ using namespace dom;
 #  include <string.h>
 #  include <stdlib.h>
 
-static MOZ_MUST_USE nsresult GetProcSelfStatmField(int aField, int64_t* aN) {
+[[nodiscard]] static nsresult GetProcSelfStatmField(int aField, int64_t* aN) {
   // There are more than two fields, but we're only interested in the first
   // two.
   static const int MAX_FIELD = 2;
@@ -87,7 +86,7 @@ static MOZ_MUST_USE nsresult GetProcSelfStatmField(int aField, int64_t* aN) {
   return NS_ERROR_FAILURE;
 }
 
-static MOZ_MUST_USE nsresult GetProcSelfSmapsPrivate(int64_t* aN) {
+[[nodiscard]] static nsresult GetProcSelfSmapsPrivate(int64_t* aN) {
   // You might be tempted to calculate USS by subtracting the "shared" value
   // from the "resident" value in /proc/<pid>/statm. But at least on Linux,
   // statm's "shared" value actually counts pages backed by files, which has
@@ -107,26 +106,26 @@ static MOZ_MUST_USE nsresult GetProcSelfSmapsPrivate(int64_t* aN) {
 }
 
 #  define HAVE_VSIZE_AND_RESIDENT_REPORTERS 1
-static MOZ_MUST_USE nsresult VsizeDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult VsizeDistinguishedAmount(int64_t* aN) {
   return GetProcSelfStatmField(0, aN);
 }
 
-static MOZ_MUST_USE nsresult ResidentDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult ResidentDistinguishedAmount(int64_t* aN) {
   return GetProcSelfStatmField(1, aN);
 }
 
-static MOZ_MUST_USE nsresult ResidentFastDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult ResidentFastDistinguishedAmount(int64_t* aN) {
   return ResidentDistinguishedAmount(aN);
 }
 
 #  define HAVE_RESIDENT_UNIQUE_REPORTER 1
-static MOZ_MUST_USE nsresult ResidentUniqueDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult ResidentUniqueDistinguishedAmount(int64_t* aN) {
   return GetProcSelfSmapsPrivate(aN);
 }
 
 #  ifdef HAVE_MALLINFO
 #    define HAVE_SYSTEM_HEAP_REPORTER 1
-static MOZ_MUST_USE nsresult SystemHeapSize(int64_t* aSizeOut) {
+[[nodiscard]] static nsresult SystemHeapSize(int64_t* aSizeOut) {
   struct mallinfo info = mallinfo();
 
   // The documentation in the glibc man page makes it sound like |uordblks|
@@ -179,7 +178,7 @@ static MOZ_MUST_USE nsresult SystemHeapSize(int64_t* aSizeOut) {
 #    define KP_RSS(kp) (kp.p_vm_rssize * getpagesize())
 #  endif
 
-static MOZ_MUST_USE nsresult GetKinfoProcSelf(KINFO_PROC* aProc) {
+[[nodiscard]] static nsresult GetKinfoProcSelf(KINFO_PROC* aProc) {
 #  if defined(__OpenBSD__) && defined(MOZ_SANDBOX)
   static LazyLogModule sPledgeLog("SandboxPledge");
   MOZ_LOG(sPledgeLog, LogLevel::Debug,
@@ -205,7 +204,7 @@ static MOZ_MUST_USE nsresult GetKinfoProcSelf(KINFO_PROC* aProc) {
 }
 
 #  define HAVE_VSIZE_AND_RESIDENT_REPORTERS 1
-static MOZ_MUST_USE nsresult VsizeDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult VsizeDistinguishedAmount(int64_t* aN) {
   KINFO_PROC proc;
   nsresult rv = GetKinfoProcSelf(&proc);
   if (NS_SUCCEEDED(rv)) {
@@ -214,7 +213,7 @@ static MOZ_MUST_USE nsresult VsizeDistinguishedAmount(int64_t* aN) {
   return rv;
 }
 
-static MOZ_MUST_USE nsresult ResidentDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult ResidentDistinguishedAmount(int64_t* aN) {
   KINFO_PROC proc;
   nsresult rv = GetKinfoProcSelf(&proc);
   if (NS_SUCCEEDED(rv)) {
@@ -223,7 +222,7 @@ static MOZ_MUST_USE nsresult ResidentDistinguishedAmount(int64_t* aN) {
   return rv;
 }
 
-static MOZ_MUST_USE nsresult ResidentFastDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult ResidentFastDistinguishedAmount(int64_t* aN) {
   return ResidentDistinguishedAmount(aN);
 }
 
@@ -231,8 +230,8 @@ static MOZ_MUST_USE nsresult ResidentFastDistinguishedAmount(int64_t* aN) {
 #    include <libutil.h>
 #    include <algorithm>
 
-static MOZ_MUST_USE nsresult GetKinfoVmentrySelf(int64_t* aPrss,
-                                                 uint64_t* aMaxreg) {
+[[nodiscard]] static nsresult GetKinfoVmentrySelf(int64_t* aPrss,
+                                                  uint64_t* aMaxreg) {
   int cnt;
   struct kinfo_vmentry* vmmap;
   struct kinfo_vmentry* kve;
@@ -261,7 +260,7 @@ static MOZ_MUST_USE nsresult GetKinfoVmentrySelf(int64_t* aPrss,
 }
 
 #    define HAVE_PRIVATE_REPORTER 1
-static MOZ_MUST_USE nsresult PrivateDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult PrivateDistinguishedAmount(int64_t* aN) {
   int64_t priv;
   nsresult rv = GetKinfoVmentrySelf(&priv, nullptr);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -270,8 +269,8 @@ static MOZ_MUST_USE nsresult PrivateDistinguishedAmount(int64_t* aN) {
 }
 
 #    define HAVE_VSIZE_MAX_CONTIGUOUS_REPORTER 1
-static MOZ_MUST_USE nsresult
-VsizeMaxContiguousDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult VsizeMaxContiguousDistinguishedAmount(
+    int64_t* aN) {
   uint64_t biggestRegion;
   nsresult rv = GetKinfoVmentrySelf(nullptr, &biggestRegion);
   if (NS_SUCCEEDED(rv)) {
@@ -334,7 +333,7 @@ static void XMappingIter(int64_t& aVsize, int64_t& aResident,
 }
 
 #  define HAVE_VSIZE_AND_RESIDENT_REPORTERS 1
-static MOZ_MUST_USE nsresult VsizeDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult VsizeDistinguishedAmount(int64_t* aN) {
   int64_t vsize, resident, shared;
   XMappingIter(vsize, resident, shared);
   if (vsize == -1) {
@@ -344,7 +343,7 @@ static MOZ_MUST_USE nsresult VsizeDistinguishedAmount(int64_t* aN) {
   return NS_OK;
 }
 
-static MOZ_MUST_USE nsresult ResidentDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult ResidentDistinguishedAmount(int64_t* aN) {
   int64_t vsize, resident, shared;
   XMappingIter(vsize, resident, shared);
   if (resident == -1) {
@@ -354,12 +353,12 @@ static MOZ_MUST_USE nsresult ResidentDistinguishedAmount(int64_t* aN) {
   return NS_OK;
 }
 
-static MOZ_MUST_USE nsresult ResidentFastDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult ResidentFastDistinguishedAmount(int64_t* aN) {
   return ResidentDistinguishedAmount(aN);
 }
 
 #  define HAVE_RESIDENT_UNIQUE_REPORTER 1
-static MOZ_MUST_USE nsresult ResidentUniqueDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult ResidentUniqueDistinguishedAmount(int64_t* aN) {
   int64_t vsize, resident, shared;
   XMappingIter(vsize, resident, shared);
   if (resident == -1) {
@@ -377,7 +376,7 @@ static MOZ_MUST_USE nsresult ResidentUniqueDistinguishedAmount(int64_t* aN) {
 #  include <mach/task.h>
 #  include <sys/sysctl.h>
 
-static MOZ_MUST_USE bool GetTaskBasicInfo(struct task_basic_info* aTi) {
+[[nodiscard]] static bool GetTaskBasicInfo(struct task_basic_info* aTi) {
   mach_msg_type_number_t count = TASK_BASIC_INFO_COUNT;
   kern_return_t kr =
       task_info(mach_task_self(), TASK_BASIC_INFO, (task_info_t)aTi, &count);
@@ -388,7 +387,7 @@ static MOZ_MUST_USE bool GetTaskBasicInfo(struct task_basic_info* aTi) {
 // absurdly high, eg. 2GB+ even at start-up.  But both 'top' and 'ps' report
 // it, so we might as well too.
 #  define HAVE_VSIZE_AND_RESIDENT_REPORTERS 1
-static MOZ_MUST_USE nsresult VsizeDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult VsizeDistinguishedAmount(int64_t* aN) {
   task_basic_info ti;
   if (!GetTaskBasicInfo(&ti)) {
     return NS_ERROR_FAILURE;
@@ -404,8 +403,8 @@ static MOZ_MUST_USE nsresult VsizeDistinguishedAmount(int64_t* aN) {
 //
 // Purging these pages can take a long time for some users (see bug 789975),
 // so we provide the option to get the RSS without purging first.
-static MOZ_MUST_USE nsresult ResidentDistinguishedAmountHelper(int64_t* aN,
-                                                               bool aDoPurge) {
+[[nodiscard]] static nsresult ResidentDistinguishedAmountHelper(int64_t* aN,
+                                                                bool aDoPurge) {
 #  ifdef HAVE_JEMALLOC_STATS
   if (aDoPurge) {
     Telemetry::AutoTimer<Telemetry::MEMORY_FREE_PURGED_PAGES_MS> timer;
@@ -421,11 +420,11 @@ static MOZ_MUST_USE nsresult ResidentDistinguishedAmountHelper(int64_t* aN,
   return NS_OK;
 }
 
-static MOZ_MUST_USE nsresult ResidentFastDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult ResidentFastDistinguishedAmount(int64_t* aN) {
   return ResidentDistinguishedAmountHelper(aN, /* doPurge = */ false);
 }
 
-static MOZ_MUST_USE nsresult ResidentDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult ResidentDistinguishedAmount(int64_t* aN) {
   return ResidentDistinguishedAmountHelper(aN, /* doPurge = */ true);
 }
 
@@ -455,7 +454,7 @@ static bool InSharedRegion(mach_vm_address_t aAddr, cpu_type_t aType) {
   return base <= aAddr && aAddr < (base + size);
 }
 
-static MOZ_MUST_USE nsresult ResidentUniqueDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult ResidentUniqueDistinguishedAmount(int64_t* aN) {
   if (!aN) {
     return NS_ERROR_FAILURE;
   }
@@ -526,7 +525,7 @@ static MOZ_MUST_USE nsresult ResidentUniqueDistinguishedAmount(int64_t* aN) {
 #  include <algorithm>
 
 #  define HAVE_VSIZE_AND_RESIDENT_REPORTERS 1
-static MOZ_MUST_USE nsresult VsizeDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult VsizeDistinguishedAmount(int64_t* aN) {
   MEMORYSTATUSEX s;
   s.dwLength = sizeof(s);
 
@@ -538,7 +537,7 @@ static MOZ_MUST_USE nsresult VsizeDistinguishedAmount(int64_t* aN) {
   return NS_OK;
 }
 
-static MOZ_MUST_USE nsresult ResidentDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult ResidentDistinguishedAmount(int64_t* aN) {
   PROCESS_MEMORY_COUNTERS pmc;
   pmc.cb = sizeof(PROCESS_MEMORY_COUNTERS);
 
@@ -550,13 +549,13 @@ static MOZ_MUST_USE nsresult ResidentDistinguishedAmount(int64_t* aN) {
   return NS_OK;
 }
 
-static MOZ_MUST_USE nsresult ResidentFastDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult ResidentFastDistinguishedAmount(int64_t* aN) {
   return ResidentDistinguishedAmount(aN);
 }
 
 #  define HAVE_RESIDENT_UNIQUE_REPORTER 1
 
-static MOZ_MUST_USE nsresult ResidentUniqueDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult ResidentUniqueDistinguishedAmount(int64_t* aN) {
   // Determine how many entries we need.
   PSAPI_WORKING_SET_INFORMATION tmp;
   DWORD tmpSize = sizeof(tmp);
@@ -602,8 +601,8 @@ static MOZ_MUST_USE nsresult ResidentUniqueDistinguishedAmount(int64_t* aN) {
 }
 
 #  define HAVE_VSIZE_MAX_CONTIGUOUS_REPORTER 1
-static MOZ_MUST_USE nsresult
-VsizeMaxContiguousDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult VsizeMaxContiguousDistinguishedAmount(
+    int64_t* aN) {
   SIZE_T biggestRegion = 0;
   MEMORY_BASIC_INFORMATION vmemInfo = {0};
   for (size_t currentAddress = 0;;) {
@@ -630,7 +629,7 @@ VsizeMaxContiguousDistinguishedAmount(int64_t* aN) {
 }
 
 #  define HAVE_PRIVATE_REPORTER 1
-static MOZ_MUST_USE nsresult PrivateDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult PrivateDistinguishedAmount(int64_t* aN) {
   PROCESS_MEMORY_COUNTERS_EX pmcex;
   pmcex.cb = sizeof(PROCESS_MEMORY_COUNTERS_EX);
 
@@ -644,59 +643,35 @@ static MOZ_MUST_USE nsresult PrivateDistinguishedAmount(int64_t* aN) {
 }
 
 #  define HAVE_SYSTEM_HEAP_REPORTER 1
-// Windows can have multiple separate heaps. During testing there were multiple
-// heaps present but the non-default ones had sizes no more than a few 10s of
-// KiBs. So we combine their sizes into a single measurement.
-static MOZ_MUST_USE nsresult SystemHeapSize(int64_t* aSizeOut) {
-  // Get the number of heaps.
-  DWORD nHeaps = GetProcessHeaps(0, nullptr);
-  NS_ENSURE_TRUE(nHeaps != 0, NS_ERROR_FAILURE);
+// Windows can have multiple separate heaps, but we should not touch non-default
+// heaps because they may be destroyed at anytime while we hold a handle.  So we
+// count only the default heap.
+[[nodiscard]] static nsresult SystemHeapSize(int64_t* aSizeOut) {
+  HANDLE heap = GetProcessHeap();
 
-  // Get handles to all heaps, checking that the number of heaps hasn't
-  // changed in the meantime.
-  UniquePtr<HANDLE[]> heaps(new HANDLE[nHeaps]);
-  DWORD nHeaps2 = GetProcessHeaps(nHeaps, heaps.get());
-  NS_ENSURE_TRUE(nHeaps2 != 0 && nHeaps2 == nHeaps, NS_ERROR_FAILURE);
+  NS_ENSURE_TRUE(HeapLock(heap), NS_ERROR_FAILURE);
 
-  // Lock and iterate over each heap to get its size.
-  int64_t heapsSize = 0;
-  for (DWORD i = 0; i < nHeaps; i++) {
-    HANDLE heap = heaps[i];
-
-    // Bug 1235982: When Control Flow Guard is enabled for the process,
-    // GetProcessHeap may return some protected heaps that are in read-only
-    // memory and thus crash in HeapLock. Ignore such heaps.
-    MEMORY_BASIC_INFORMATION mbi = {0};
-    if (VirtualQuery(heap, &mbi, sizeof(mbi)) && mbi.Protect == PAGE_READONLY) {
-      continue;
+  int64_t heapSize = 0;
+  PROCESS_HEAP_ENTRY entry;
+  entry.lpData = nullptr;
+  while (HeapWalk(heap, &entry)) {
+    // We don't count entry.cbOverhead, because we just want to measure the
+    // space available to the program.
+    if (entry.wFlags & PROCESS_HEAP_ENTRY_BUSY) {
+      heapSize += entry.cbData;
     }
-
-    NS_ENSURE_TRUE(HeapLock(heap), NS_ERROR_FAILURE);
-
-    int64_t heapSize = 0;
-    PROCESS_HEAP_ENTRY entry;
-    entry.lpData = nullptr;
-    while (HeapWalk(heap, &entry)) {
-      // We don't count entry.cbOverhead, because we just want to measure the
-      // space available to the program.
-      if (entry.wFlags & PROCESS_HEAP_ENTRY_BUSY) {
-        heapSize += entry.cbData;
-      }
-    }
-
-    // Check this result only after unlocking the heap, so that we don't leave
-    // the heap locked if there was an error.
-    DWORD lastError = GetLastError();
-
-    // I have no idea how things would proceed if unlocking this heap failed...
-    NS_ENSURE_TRUE(HeapUnlock(heap), NS_ERROR_FAILURE);
-
-    NS_ENSURE_TRUE(lastError == ERROR_NO_MORE_ITEMS, NS_ERROR_FAILURE);
-
-    heapsSize += heapSize;
   }
 
-  *aSizeOut = heapsSize;
+  // Check this result only after unlocking the heap, so that we don't leave
+  // the heap locked if there was an error.
+  DWORD lastError = GetLastError();
+
+  // I have no idea how things would proceed if unlocking this heap failed...
+  NS_ENSURE_TRUE(HeapUnlock(heap), NS_ERROR_FAILURE);
+
+  NS_ENSURE_TRUE(lastError == ERROR_NO_MORE_ITEMS, NS_ERROR_FAILURE);
+
+  *aSizeOut = heapSize;
   return NS_OK;
 }
 
@@ -965,7 +940,7 @@ NS_IMPL_ISUPPORTS(PrivateReporter, nsIMemoryReporter)
 
 #ifdef HAVE_VSIZE_AND_RESIDENT_REPORTERS
 class VsizeReporter final : public nsIMemoryReporter {
-  ~VsizeReporter() {}
+  ~VsizeReporter() = default;
 
  public:
   NS_DECL_ISUPPORTS
@@ -992,7 +967,7 @@ class VsizeReporter final : public nsIMemoryReporter {
 NS_IMPL_ISUPPORTS(VsizeReporter, nsIMemoryReporter)
 
 class ResidentReporter final : public nsIMemoryReporter {
-  ~ResidentReporter() {}
+  ~ResidentReporter() = default;
 
  public:
   NS_DECL_ISUPPORTS
@@ -1021,7 +996,7 @@ NS_IMPL_ISUPPORTS(ResidentReporter, nsIMemoryReporter)
 
 #ifdef HAVE_RESIDENT_UNIQUE_REPORTER
 class ResidentUniqueReporter final : public nsIMemoryReporter {
-  ~ResidentUniqueReporter() {}
+  ~ResidentUniqueReporter() = default;
 
  public:
   NS_DECL_ISUPPORTS
@@ -1049,7 +1024,7 @@ NS_IMPL_ISUPPORTS(ResidentUniqueReporter, nsIMemoryReporter)
 #ifdef HAVE_SYSTEM_HEAP_REPORTER
 
 class SystemHeapReporter final : public nsIMemoryReporter {
-  ~SystemHeapReporter() {}
+  ~SystemHeapReporter() = default;
 
  public:
   NS_DECL_ISUPPORTS
@@ -1071,7 +1046,6 @@ class SystemHeapReporter final : public nsIMemoryReporter {
   }
 };
 NS_IMPL_ISUPPORTS(SystemHeapReporter, nsIMemoryReporter)
-
 #endif  // HAVE_SYSTEM_HEAP_REPORTER
 
 #ifdef XP_UNIX
@@ -1080,7 +1054,7 @@ NS_IMPL_ISUPPORTS(SystemHeapReporter, nsIMemoryReporter)
 
 #  define HAVE_RESIDENT_PEAK_REPORTER 1
 
-static MOZ_MUST_USE nsresult ResidentPeakDistinguishedAmount(int64_t* aN) {
+[[nodiscard]] static nsresult ResidentPeakDistinguishedAmount(int64_t* aN) {
   struct rusage usage;
   if (0 == getrusage(RUSAGE_SELF, &usage)) {
     // The units for ru_maxrrs:
@@ -1103,7 +1077,7 @@ static MOZ_MUST_USE nsresult ResidentPeakDistinguishedAmount(int64_t* aN) {
 }
 
 class ResidentPeakReporter final : public nsIMemoryReporter {
-  ~ResidentPeakReporter() {}
+  ~ResidentPeakReporter() = default;
 
  public:
   NS_DECL_ISUPPORTS
@@ -1124,7 +1098,7 @@ NS_IMPL_ISUPPORTS(ResidentPeakReporter, nsIMemoryReporter)
 #  define HAVE_PAGE_FAULT_REPORTERS 1
 
 class PageFaultsSoftReporter final : public nsIMemoryReporter {
-  ~PageFaultsSoftReporter() {}
+  ~PageFaultsSoftReporter() = default;
 
  public:
   NS_DECL_ISUPPORTS
@@ -1154,8 +1128,8 @@ class PageFaultsSoftReporter final : public nsIMemoryReporter {
 };
 NS_IMPL_ISUPPORTS(PageFaultsSoftReporter, nsIMemoryReporter)
 
-static MOZ_MUST_USE nsresult
-PageFaultsHardDistinguishedAmount(int64_t* aAmount) {
+[[nodiscard]] static nsresult
+    PageFaultsHardDistinguishedAmount(int64_t* aAmount) {
   struct rusage usage;
   int err = getrusage(RUSAGE_SELF, &usage);
   if (err != 0) {
@@ -1166,7 +1140,7 @@ PageFaultsHardDistinguishedAmount(int64_t* aAmount) {
 }
 
 class PageFaultsHardReporter final : public nsIMemoryReporter {
-  ~PageFaultsHardReporter() {}
+  ~PageFaultsHardReporter() = default;
 
  public:
   NS_DECL_ISUPPORTS
@@ -1219,7 +1193,7 @@ static int64_t HeapOverheadFraction(jemalloc_stats_t* aStats) {
 }
 
 class JemallocHeapReporter final : public nsIMemoryReporter {
-  ~JemallocHeapReporter() {}
+  ~JemallocHeapReporter() = default;
 
  public:
   NS_DECL_ISUPPORTS
@@ -1303,7 +1277,7 @@ NS_IMPL_ISUPPORTS(JemallocHeapReporter, nsIMemoryReporter)
 class AtomTablesReporter final : public nsIMemoryReporter {
   MOZ_DEFINE_MALLOC_SIZE_OF(MallocSizeOf)
 
-  ~AtomTablesReporter() {}
+  ~AtomTablesReporter() = default;
 
  public:
   NS_DECL_ISUPPORTS
@@ -1492,7 +1466,7 @@ NS_IMPL_ISUPPORTS(ThreadsReporter, nsIMemoryReporter)
 class DeadlockDetectorReporter final : public nsIMemoryReporter {
   MOZ_DEFINE_MALLOC_SIZE_OF(MallocSizeOf)
 
-  ~DeadlockDetectorReporter() {}
+  ~DeadlockDetectorReporter() = default;
 
  public:
   NS_DECL_ISUPPORTS
@@ -1553,7 +1527,7 @@ class DMDReporter final : public nsIMemoryReporter {
   }
 
  private:
-  ~DMDReporter() {}
+  ~DMDReporter() = default;
 };
 NS_IMPL_ISUPPORTS(DMDReporter, nsIMemoryReporter)
 
@@ -2437,8 +2411,8 @@ nsMemoryReporterManager::GetHeapOverheadFraction(int64_t* aAmount) {
 #endif
 }
 
-static MOZ_MUST_USE nsresult GetInfallibleAmount(InfallibleAmountFn aAmountFn,
-                                                 int64_t* aAmount) {
+[[nodiscard]] static nsresult GetInfallibleAmount(InfallibleAmountFn aAmountFn,
+                                                  int64_t* aAmount) {
   if (aAmountFn) {
     *aAmount = aAmountFn();
     return NS_OK;
