@@ -16,47 +16,44 @@
 #include "nsPointerHashKeys.h"
 #include "mozilla/Likely.h"
 #include "mozilla/MemoryReporting.h"
+#include "mozilla/Maybe.h"
 #include "mozilla/ChaosMode.h"
 
 using namespace mozilla;
 
-#ifdef DEBUG
+#ifdef MOZ_HASH_TABLE_CHECKS_ENABLED
 
-class AutoReadOp
-{
+class AutoReadOp {
   Checker& mChk;
-public:
+
+ public:
   explicit AutoReadOp(Checker& aChk) : mChk(aChk) { mChk.StartReadOp(); }
   ~AutoReadOp() { mChk.EndReadOp(); }
 };
 
-class AutoWriteOp
-{
+class AutoWriteOp {
   Checker& mChk;
-public:
+
+ public:
   explicit AutoWriteOp(Checker& aChk) : mChk(aChk) { mChk.StartWriteOp(); }
   ~AutoWriteOp() { mChk.EndWriteOp(); }
 };
 
-class AutoIteratorRemovalOp
-{
+class AutoIteratorRemovalOp {
   Checker& mChk;
-public:
-  explicit AutoIteratorRemovalOp(Checker& aChk)
-    : mChk(aChk)
-  {
+
+ public:
+  explicit AutoIteratorRemovalOp(Checker& aChk) : mChk(aChk) {
     mChk.StartIteratorRemovalOp();
   }
   ~AutoIteratorRemovalOp() { mChk.EndIteratorRemovalOp(); }
 };
 
-class AutoDestructorOp
-{
+class AutoDestructorOp {
   Checker& mChk;
-public:
-  explicit AutoDestructorOp(Checker& aChk)
-    : mChk(aChk)
-  {
+
+ public:
+  explicit AutoDestructorOp(Checker& aChk) : mChk(aChk) {
     mChk.StartDestructorOp();
   }
   ~AutoDestructorOp() { mChk.EndDestructorOp(); }
@@ -64,29 +61,27 @@ public:
 
 #endif
 
-/* static */ PLDHashNumber
-PLDHashTable::HashStringKey(const void* aKey)
-{
+/* static */
+PLDHashNumber PLDHashTable::HashStringKey(const void* aKey) {
   return HashString(static_cast<const char*>(aKey));
 }
 
-/* static */ PLDHashNumber
-PLDHashTable::HashVoidPtrKeyStub(const void* aKey)
-{
+/* static */
+PLDHashNumber PLDHashTable::HashVoidPtrKeyStub(const void* aKey) {
   return nsPtrHashKey<void>::HashKey(aKey);
 }
 
-/* static */ bool
-PLDHashTable::MatchEntryStub(const PLDHashEntryHdr* aEntry, const void* aKey)
-{
+/* static */
+bool PLDHashTable::MatchEntryStub(const PLDHashEntryHdr* aEntry,
+                                  const void* aKey) {
   const PLDHashEntryStub* stub = (const PLDHashEntryStub*)aEntry;
 
   return stub->key == aKey;
 }
 
-/* static */ bool
-PLDHashTable::MatchStringKey(const PLDHashEntryHdr* aEntry, const void* aKey)
-{
+/* static */
+bool PLDHashTable::MatchStringKey(const PLDHashEntryHdr* aEntry,
+                                  const void* aKey) {
   const PLDHashEntryStub* stub = (const PLDHashEntryStub*)aEntry;
 
   // XXX tolerate null keys on account of sloppy Mozilla callers.
@@ -95,40 +90,33 @@ PLDHashTable::MatchStringKey(const PLDHashEntryHdr* aEntry, const void* aKey)
           strcmp((const char*)stub->key, (const char*)aKey) == 0);
 }
 
-/* static */ void
-PLDHashTable::MoveEntryStub(PLDHashTable* aTable,
-                            const PLDHashEntryHdr* aFrom,
-                            PLDHashEntryHdr* aTo)
-{
+/* static */
+void PLDHashTable::MoveEntryStub(PLDHashTable* aTable,
+                                 const PLDHashEntryHdr* aFrom,
+                                 PLDHashEntryHdr* aTo) {
   memcpy(aTo, aFrom, aTable->mEntrySize);
 }
 
-/* static */ void
-PLDHashTable::ClearEntryStub(PLDHashTable* aTable, PLDHashEntryHdr* aEntry)
-{
+/* static */
+void PLDHashTable::ClearEntryStub(PLDHashTable* aTable,
+                                  PLDHashEntryHdr* aEntry) {
   memset(aEntry, 0, aTable->mEntrySize);
 }
 
 static const PLDHashTableOps gStubOps = {
-  PLDHashTable::HashVoidPtrKeyStub,
-  PLDHashTable::MatchEntryStub,
-  PLDHashTable::MoveEntryStub,
-  PLDHashTable::ClearEntryStub,
-  nullptr
-};
+    PLDHashTable::HashVoidPtrKeyStub, PLDHashTable::MatchEntryStub,
+    PLDHashTable::MoveEntryStub, PLDHashTable::ClearEntryStub, nullptr};
 
-/* static */ const PLDHashTableOps*
-PLDHashTable::StubOps()
-{
+/* static */ const PLDHashTableOps* PLDHashTable::StubOps() {
   return &gStubOps;
 }
 
-static bool
-SizeOfEntryStore(uint32_t aCapacity, uint32_t aEntrySize, uint32_t* aNbytes)
-{
-  uint64_t nbytes64 = uint64_t(aCapacity) * uint64_t(aEntrySize);
-  *aNbytes = aCapacity * aEntrySize;
-  return uint64_t(*aNbytes) == nbytes64;   // returns false on overflow
+static bool SizeOfEntryStore(uint32_t aCapacity, uint32_t aEntrySize,
+                             uint32_t* aNbytes) {
+  uint32_t slotSize = aEntrySize + sizeof(PLDHashNumber);
+  uint64_t nbytes64 = uint64_t(aCapacity) * uint64_t(slotSize);
+  *aNbytes = aCapacity * slotSize;
+  return uint64_t(*aNbytes) == nbytes64;  // returns false on overflow
 }
 
 // Compute max and min load numbers (entry counts). We have a secondary max
@@ -136,20 +124,14 @@ SizeOfEntryStore(uint32_t aCapacity, uint32_t aEntrySize, uint32_t* aNbytes)
 // (i.e. if ChangeTable() fails). The table slows down drastically if the
 // secondary max is too close to 1, but 0.96875 gives only a slight slowdown
 // while allowing 1.3x more elements.
-static inline uint32_t
-MaxLoad(uint32_t aCapacity)
-{
+static inline uint32_t MaxLoad(uint32_t aCapacity) {
   return aCapacity - (aCapacity >> 2);  // == aCapacity * 0.75
 }
-static inline uint32_t
-MaxLoadOnGrowthFailure(uint32_t aCapacity)
-{
+static inline uint32_t MaxLoadOnGrowthFailure(uint32_t aCapacity) {
   return aCapacity - (aCapacity >> 5);  // == aCapacity * 0.96875
 }
-static inline uint32_t
-MinLoad(uint32_t aCapacity)
-{
-  return aCapacity >> 2;                // == aCapacity * 0.25
+static inline uint32_t MinLoad(uint32_t aCapacity) {
+  return aCapacity >> 2;  // == aCapacity * 0.25
 }
 
 // Compute the minimum capacity (and the Log2 of that capacity) for a table
@@ -157,13 +139,14 @@ MinLoad(uint32_t aCapacity)
 // - table must be at most 75% full;
 // - capacity must be a power of two;
 // - capacity cannot be too small.
-static inline void
-BestCapacity(uint32_t aLength, uint32_t* aCapacityOut,
-             uint32_t* aLog2CapacityOut)
-{
+static inline void BestCapacity(uint32_t aLength, uint32_t* aCapacityOut,
+                                uint32_t* aLog2CapacityOut) {
+  // Callers should ensure this is true.
+  MOZ_ASSERT(aLength <= PLDHashTable::kMaxInitialLength);
+
   // Compute the smallest capacity allowing |aLength| elements to be inserted
   // without rehashing.
-  uint32_t capacity = (aLength * 4 + (3 - 1)) / 3; // == ceil(aLength * 4 / 3)
+  uint32_t capacity = (aLength * 4 + (3 - 1)) / 3;  // == ceil(aLength * 4 / 3)
   if (capacity < PLDHashTable::kMinCapacity) {
     capacity = PLDHashTable::kMinCapacity;
   }
@@ -178,8 +161,7 @@ BestCapacity(uint32_t aLength, uint32_t* aCapacityOut,
 }
 
 /* static */ MOZ_ALWAYS_INLINE uint32_t
-PLDHashTable::HashShift(uint32_t aEntrySize, uint32_t aLength)
-{
+PLDHashTable::HashShift(uint32_t aEntrySize, uint32_t aLength) {
   if (aLength > kMaxInitialLength) {
     MOZ_CRASH("Initial length is too large");
   }
@@ -193,22 +175,18 @@ PLDHashTable::HashShift(uint32_t aEntrySize, uint32_t aLength)
   }
 
   // Compute the hashShift value.
-  return kHashBits - log2;
+  return kPLDHashNumberBits - log2;
 }
 
 PLDHashTable::PLDHashTable(const PLDHashTableOps* aOps, uint32_t aEntrySize,
                            uint32_t aLength)
-  : mOps(aOps)
-  , mEntryStore()
-  , mGeneration(0)
-  , mHashShift(HashShift(aEntrySize, aLength))
-  , mEntrySize(aEntrySize)
-  , mEntryCount(0)
-  , mRemovedCount(0)
-#ifdef DEBUG
-  , mChecker()
-#endif
-{
+    : mOps(aOps),
+      mEntryStore(),
+      mGeneration(0),
+      mHashShift(HashShift(aEntrySize, aLength)),
+      mEntrySize(aEntrySize),
+      mEntryCount(0),
+      mRemovedCount(0) {
   // An entry size greater than 0xff is unlikely, but let's check anyway. If
   // you hit this, your hashtable would waste lots of space for unused entries
   // and you should change your hash table's entries to pointers.
@@ -217,9 +195,7 @@ PLDHashTable::PLDHashTable(const PLDHashTableOps* aOps, uint32_t aEntrySize,
   }
 }
 
-PLDHashTable&
-PLDHashTable::operator=(PLDHashTable&& aOther)
-{
+PLDHashTable& PLDHashTable::operator=(PLDHashTable&& aOther) {
   if (this == &aOther) {
     return *this;
   }
@@ -228,44 +204,43 @@ PLDHashTable::operator=(PLDHashTable&& aOther)
   // conceptually part of the type -- indeed, if PLDHashTable was a templated
   // type like nsTHashtable, they *would* be part of the type -- so it only
   // makes sense to assign in cases where they match.
-  MOZ_RELEASE_ASSERT(mOps == aOther.mOps);
-  MOZ_RELEASE_ASSERT(mEntrySize == aOther.mEntrySize);
+  MOZ_RELEASE_ASSERT(mOps == aOther.mOps || !mOps);
+  MOZ_RELEASE_ASSERT(mEntrySize == aOther.mEntrySize || !mEntrySize);
 
   // Reconstruct |this|.
+  const PLDHashTableOps* ops = aOther.mOps;
   this->~PLDHashTable();
-  new (KnownNotNull, this) PLDHashTable(aOther.mOps, aOther.mEntrySize, 0);
+  new (KnownNotNull, this) PLDHashTable(ops, aOther.mEntrySize, 0);
 
   // Move non-const pieces over.
-  mHashShift = Move(aOther.mHashShift);
-  mEntryCount = Move(aOther.mEntryCount);
-  mRemovedCount = Move(aOther.mRemovedCount);
+  mHashShift = std::move(aOther.mHashShift);
+  mEntryCount = std::move(aOther.mEntryCount);
+  mRemovedCount = std::move(aOther.mRemovedCount);
   mEntryStore.Set(aOther.mEntryStore.Get(), &mGeneration);
-#ifdef DEBUG
-  mChecker = Move(aOther.mChecker);
+#ifdef MOZ_HASH_TABLE_CHECKS_ENABLED
+  mChecker = std::move(aOther.mChecker);
 #endif
 
-  // Clear up |aOther| so its destruction will be a no-op.
+  // Clear up |aOther| so its destruction will be a no-op and it reports being
+  // empty.
   {
-#ifdef DEBUG
+#ifdef MOZ_HASH_TABLE_CHECKS_ENABLED
     AutoDestructorOp op(mChecker);
 #endif
+    aOther.mEntryCount = 0;
     aOther.mEntryStore.Set(nullptr, &aOther.mGeneration);
   }
 
   return *this;
 }
 
-PLDHashNumber
-PLDHashTable::Hash1(PLDHashNumber aHash0)
-{
+PLDHashNumber PLDHashTable::Hash1(PLDHashNumber aHash0) const {
   return aHash0 >> mHashShift;
 }
 
-void
-PLDHashTable::Hash2(PLDHashNumber aHash0,
-                    uint32_t& aHash2Out, uint32_t& aSizeMaskOut)
-{
-  uint32_t sizeLog2 = kHashBits - mHashShift;
+void PLDHashTable::Hash2(PLDHashNumber aHash0, uint32_t& aHash2Out,
+                         uint32_t& aSizeMaskOut) const {
+  uint32_t sizeLog2 = kPLDHashNumberBits - mHashShift;
   uint32_t sizeMask = (PLDHashNumber(1) << sizeLog2) - 1;
   aSizeMaskOut = sizeMask;
 
@@ -292,23 +267,18 @@ PLDHashTable::Hash2(PLDHashNumber aHash0,
 // should not hurt the hash function's effectiveness much.
 
 // Match an entry's mKeyHash against an unstored one computed from a key.
-/* static */ bool
-PLDHashTable::MatchEntryKeyhash(PLDHashEntryHdr* aEntry, PLDHashNumber aKeyHash)
-{
-  return (aEntry->mKeyHash & ~kCollisionFlag) == aKeyHash;
+/* static */
+bool PLDHashTable::MatchSlotKeyhash(Slot& aSlot, const PLDHashNumber aKeyHash) {
+  return (aSlot.KeyHash() & ~kCollisionFlag) == aKeyHash;
 }
 
 // Compute the address of the indexed entry in table.
-PLDHashEntryHdr*
-PLDHashTable::AddressEntry(uint32_t aIndex)
-{
-  return reinterpret_cast<PLDHashEntryHdr*>(
-    mEntryStore.Get() + aIndex * mEntrySize);
+auto PLDHashTable::SlotForIndex(uint32_t aIndex) const -> Slot {
+  return mEntryStore.SlotForIndex(aIndex, mEntrySize, CapacityFromHashShift());
 }
 
-PLDHashTable::~PLDHashTable()
-{
-#ifdef DEBUG
+PLDHashTable::~PLDHashTable() {
+#ifdef MOZ_HASH_TABLE_CHECKS_ENABLED
   AutoDestructorOp op(mChecker);
 #endif
 
@@ -317,22 +287,16 @@ PLDHashTable::~PLDHashTable()
   }
 
   // Clear any remaining live entries.
-  char* entryAddr = mEntryStore.Get();
-  char* entryLimit = entryAddr + Capacity() * mEntrySize;
-  while (entryAddr < entryLimit) {
-    PLDHashEntryHdr* entry = (PLDHashEntryHdr*)entryAddr;
-    if (EntryIsLive(entry)) {
-      mOps->clearEntry(this, entry);
+  mEntryStore.ForEachSlot(Capacity(), mEntrySize, [&](const Slot& aSlot) {
+    if (aSlot.IsLive()) {
+      mOps->clearEntry(this, aSlot.ToEntry());
     }
-    entryAddr += mEntrySize;
-  }
+  });
 
   // Entry storage is freed last, by ~EntryStore().
 }
 
-void
-PLDHashTable::ClearAndPrepareForLength(uint32_t aLength)
-{
+void PLDHashTable::ClearAndPrepareForLength(uint32_t aLength) {
   // Get these values before the destructor clobbers them.
   const PLDHashTableOps* ops = mOps;
   uint32_t entrySize = mEntrySize;
@@ -341,39 +305,38 @@ PLDHashTable::ClearAndPrepareForLength(uint32_t aLength)
   new (KnownNotNull, this) PLDHashTable(ops, entrySize, aLength);
 }
 
-void
-PLDHashTable::Clear()
-{
-  ClearAndPrepareForLength(kDefaultInitialLength);
-}
+void PLDHashTable::Clear() { ClearAndPrepareForLength(kDefaultInitialLength); }
 
 // If |Reason| is |ForAdd|, the return value is always non-null and it may be
 // a previously-removed entry. If |Reason| is |ForSearchOrRemove|, the return
 // value is null on a miss, and will never be a previously-removed entry on a
 // hit. This distinction is a bit grotty but this function is hot enough that
-// these differences are worthwhile.
-template <PLDHashTable::SearchReason Reason>
-PLDHashEntryHdr* NS_FASTCALL
-PLDHashTable::SearchTable(const void* aKey, PLDHashNumber aKeyHash)
-{
+// these differences are worthwhile. (It's also hot enough that
+// MOZ_ALWAYS_INLINE makes a significant difference.)
+template <PLDHashTable::SearchReason Reason, typename Success, typename Failure>
+MOZ_ALWAYS_INLINE auto PLDHashTable::SearchTable(const void* aKey,
+                                                 PLDHashNumber aKeyHash,
+                                                 Success&& aSuccess,
+                                                 Failure&& aFailure) const {
   MOZ_ASSERT(mEntryStore.Get());
-  NS_ASSERTION(!(aKeyHash & kCollisionFlag),
-               "!(aKeyHash & kCollisionFlag)");
+  NS_ASSERTION(!(aKeyHash & kCollisionFlag), "!(aKeyHash & kCollisionFlag)");
 
   // Compute the primary hash address.
   PLDHashNumber hash1 = Hash1(aKeyHash);
-  PLDHashEntryHdr* entry = AddressEntry(hash1);
+  Slot slot = SlotForIndex(hash1);
 
   // Miss: return space for a new entry.
-  if (EntryIsFree(entry)) {
-    return (Reason == ForAdd) ? entry : nullptr;
+  if (slot.IsFree()) {
+    return (Reason == ForAdd) ? aSuccess(slot) : aFailure();
   }
 
   // Hit: return entry.
   PLDHashMatchEntry matchEntry = mOps->matchEntry;
-  if (MatchEntryKeyhash(entry, aKeyHash) &&
-      matchEntry(entry, aKey)) {
-    return entry;
+  if (MatchSlotKeyhash(slot, aKeyHash)) {
+    PLDHashEntryHdr* e = slot.ToEntry();
+    if (matchEntry(e, aKey)) {
+      return aSuccess(slot);
+    }
   }
 
   // Collision: double hash.
@@ -381,36 +344,40 @@ PLDHashTable::SearchTable(const void* aKey, PLDHashNumber aKeyHash)
   uint32_t sizeMask;
   Hash2(aKeyHash, hash2, sizeMask);
 
-  // Save the first removed entry pointer so Add() can recycle it. (Only used
+  // Save the first removed entry slot so Add() can recycle it. (Only used
   // if Reason==ForAdd.)
-  PLDHashEntryHdr* firstRemoved = nullptr;
+  Maybe<Slot> firstRemoved;
 
   for (;;) {
     if (Reason == ForAdd && !firstRemoved) {
-      if (MOZ_UNLIKELY(EntryIsRemoved(entry))) {
-        firstRemoved = entry;
+      if (MOZ_UNLIKELY(slot.IsRemoved())) {
+        firstRemoved.emplace(slot);
       } else {
-        entry->mKeyHash |= kCollisionFlag;
+        slot.MarkColliding();
       }
     }
 
     hash1 -= hash2;
     hash1 &= sizeMask;
 
-    entry = AddressEntry(hash1);
-    if (EntryIsFree(entry)) {
-      return (Reason == ForAdd) ? (firstRemoved ? firstRemoved : entry)
-                                : nullptr;
+    slot = SlotForIndex(hash1);
+    if (slot.IsFree()) {
+      if (Reason != ForAdd) {
+        return aFailure();
+      }
+      return aSuccess(firstRemoved.refOr(slot));
     }
 
-    if (MatchEntryKeyhash(entry, aKeyHash) &&
-        matchEntry(entry, aKey)) {
-      return entry;
+    if (MatchSlotKeyhash(slot, aKeyHash)) {
+      PLDHashEntryHdr* e = slot.ToEntry();
+      if (matchEntry(e, aKey)) {
+        return aSuccess(slot);
+      }
     }
   }
 
   // NOTREACHED
-  return nullptr;
+  return aFailure();
 }
 
 // This is a copy of SearchTable(), used by ChangeTable(), hardcoded to
@@ -420,20 +387,18 @@ PLDHashTable::SearchTable(const void* aKey, PLDHashNumber aKeyHash)
 //      structure.
 // Avoiding the need for |aKey| means we can avoid needing a way to map entries
 // to keys, which means callers can use complex key types more easily.
-MOZ_ALWAYS_INLINE PLDHashEntryHdr*
-PLDHashTable::FindFreeEntry(PLDHashNumber aKeyHash)
-{
+MOZ_ALWAYS_INLINE auto PLDHashTable::FindFreeSlot(PLDHashNumber aKeyHash) const
+    -> Slot {
   MOZ_ASSERT(mEntryStore.Get());
-  NS_ASSERTION(!(aKeyHash & kCollisionFlag),
-               "!(aKeyHash & kCollisionFlag)");
+  NS_ASSERTION(!(aKeyHash & kCollisionFlag), "!(aKeyHash & kCollisionFlag)");
 
   // Compute the primary hash address.
   PLDHashNumber hash1 = Hash1(aKeyHash);
-  PLDHashEntryHdr* entry = AddressEntry(hash1);
+  Slot slot = SlotForIndex(hash1);
 
   // Miss: return space for a new entry.
-  if (EntryIsFree(entry)) {
-    return entry;
+  if (slot.IsFree()) {
+    return slot;
   }
 
   // Collision: double hash.
@@ -442,29 +407,26 @@ PLDHashTable::FindFreeEntry(PLDHashNumber aKeyHash)
   Hash2(aKeyHash, hash2, sizeMask);
 
   for (;;) {
-    NS_ASSERTION(!EntryIsRemoved(entry),
-                 "!EntryIsRemoved(entry)");
-    entry->mKeyHash |= kCollisionFlag;
+    MOZ_ASSERT(!slot.IsRemoved());
+    slot.MarkColliding();
 
     hash1 -= hash2;
     hash1 &= sizeMask;
 
-    entry = AddressEntry(hash1);
-    if (EntryIsFree(entry)) {
-      return entry;
+    slot = SlotForIndex(hash1);
+    if (slot.IsFree()) {
+      return slot;
     }
   }
 
   // NOTREACHED
 }
 
-bool
-PLDHashTable::ChangeTable(int32_t aDeltaLog2)
-{
+bool PLDHashTable::ChangeTable(int32_t aDeltaLog2) {
   MOZ_ASSERT(mEntryStore.Get());
 
   // Look, but don't touch, until we succeed in getting new entry store.
-  int32_t oldLog2 = kHashBits - mHashShift;
+  int32_t oldLog2 = kPLDHashNumberBits - mHashShift;
   int32_t newLog2 = oldLog2 + aDeltaLog2;
   uint32_t newCapacity = 1u << newLog2;
   if (newCapacity > kMaxCapacity) {
@@ -473,51 +435,45 @@ PLDHashTable::ChangeTable(int32_t aDeltaLog2)
 
   uint32_t nbytes;
   if (!SizeOfEntryStore(newCapacity, mEntrySize, &nbytes)) {
-    return false;   // overflowed
+    return false;  // overflowed
   }
 
-  char* newEntryStore = (char*)malloc(nbytes);
+  char* newEntryStore = (char*)calloc(1, nbytes);
   if (!newEntryStore) {
     return false;
   }
 
   // We can't fail from here on, so update table parameters.
-  mHashShift = kHashBits - newLog2;
+  mHashShift = kPLDHashNumberBits - newLog2;
   mRemovedCount = 0;
 
   // Assign the new entry store to table.
-  memset(newEntryStore, 0, nbytes);
-  char* oldEntryStore;
-  char* oldEntryAddr;
-  oldEntryAddr = oldEntryStore = mEntryStore.Get();
+  char* oldEntryStore = mEntryStore.Get();
   mEntryStore.Set(newEntryStore, &mGeneration);
   PLDHashMoveEntry moveEntry = mOps->moveEntry;
 
   // Copy only live entries, leaving removed ones behind.
   uint32_t oldCapacity = 1u << oldLog2;
-  for (uint32_t i = 0; i < oldCapacity; ++i) {
-    PLDHashEntryHdr* oldEntry = (PLDHashEntryHdr*)oldEntryAddr;
-    if (EntryIsLive(oldEntry)) {
-      oldEntry->mKeyHash &= ~kCollisionFlag;
-      PLDHashEntryHdr* newEntry = FindFreeEntry(oldEntry->mKeyHash);
-      NS_ASSERTION(EntryIsFree(newEntry), "EntryIsFree(newEntry)");
-      moveEntry(this, oldEntry, newEntry);
-      newEntry->mKeyHash = oldEntry->mKeyHash;
-    }
-    oldEntryAddr += mEntrySize;
-  }
+  EntryStore::ForEachSlot(
+      oldEntryStore, oldCapacity, mEntrySize, [&](const Slot& slot) {
+        if (slot.IsLive()) {
+          const PLDHashNumber key = slot.KeyHash() & ~kCollisionFlag;
+          Slot newSlot = FindFreeSlot(key);
+          MOZ_ASSERT(newSlot.IsFree());
+          moveEntry(this, slot.ToEntry(), newSlot.ToEntry());
+          newSlot.SetKeyHash(key);
+        }
+      });
 
   free(oldEntryStore);
   return true;
 }
 
 MOZ_ALWAYS_INLINE PLDHashNumber
-PLDHashTable::ComputeKeyHash(const void* aKey)
-{
+PLDHashTable::ComputeKeyHash(const void* aKey) const {
   MOZ_ASSERT(mEntryStore.Get());
 
-  PLDHashNumber keyHash = mOps->hashKey(aKey);
-  keyHash *= kGoldenRatio;
+  PLDHashNumber keyHash = mozilla::ScrambleHashCode(mOps->hashKey(aKey));
 
   // Avoid 0 and 1 hash codes, they indicate free and removed entries.
   if (keyHash < 2) {
@@ -528,24 +484,24 @@ PLDHashTable::ComputeKeyHash(const void* aKey)
   return keyHash;
 }
 
-PLDHashEntryHdr*
-PLDHashTable::Search(const void* aKey)
-{
-#ifdef DEBUG
+PLDHashEntryHdr* PLDHashTable::Search(const void* aKey) const {
+#ifdef MOZ_HASH_TABLE_CHECKS_ENABLED
   AutoReadOp op(mChecker);
 #endif
 
-  PLDHashEntryHdr* entry = mEntryStore.Get()
-                         ? SearchTable<ForSearchOrRemove>(aKey,
-                                                          ComputeKeyHash(aKey))
-                         : nullptr;
-  return entry;
+  if (!mEntryStore.Get()) {
+    return nullptr;
+  }
+
+  return SearchTable<ForSearchOrRemove>(
+      aKey, ComputeKeyHash(aKey),
+      [&](Slot& slot) -> PLDHashEntryHdr* { return slot.ToEntry(); },
+      [&]() -> PLDHashEntryHdr* { return nullptr; });
 }
 
-PLDHashEntryHdr*
-PLDHashTable::Add(const void* aKey, const mozilla::fallible_t&)
-{
-#ifdef DEBUG
+PLDHashEntryHdr* PLDHashTable::Add(const void* aKey,
+                                   const mozilla::fallible_t&) {
+#ifdef MOZ_HASH_TABLE_CHECKS_ENABLED
   AutoWriteOp op(mChecker);
 #endif
 
@@ -553,13 +509,12 @@ PLDHashTable::Add(const void* aKey, const mozilla::fallible_t&)
   if (!mEntryStore.Get()) {
     uint32_t nbytes;
     // We already checked this in the constructor, so it must still be true.
-    MOZ_RELEASE_ASSERT(SizeOfEntryStore(CapacityFromHashShift(), mEntrySize,
-                                        &nbytes));
-    mEntryStore.Set((char*)malloc(nbytes), &mGeneration);
+    MOZ_RELEASE_ASSERT(
+        SizeOfEntryStore(CapacityFromHashShift(), mEntrySize, &nbytes));
+    mEntryStore.Set((char*)calloc(1, nbytes), &mGeneration);
     if (!mEntryStore.Get()) {
       return nullptr;
     }
-    memset(mEntryStore.Get(), 0, nbytes);
   }
 
   // If alpha is >= .75, grow or compress the table. If aKey is already in the
@@ -586,32 +541,35 @@ PLDHashTable::Add(const void* aKey, const mozilla::fallible_t&)
   // Look for entry after possibly growing, so we don't have to add it,
   // then skip it while growing the table and re-add it after.
   PLDHashNumber keyHash = ComputeKeyHash(aKey);
-  PLDHashEntryHdr* entry = SearchTable<ForAdd>(aKey, keyHash);
-  if (!EntryIsLive(entry)) {
-    // Initialize the entry, indicating that it's no longer free.
-    if (EntryIsRemoved(entry)) {
+  Slot slot = SearchTable<ForAdd>(
+      aKey, keyHash, [&](Slot& found) -> Slot { return found; },
+      [&]() -> Slot {
+        MOZ_CRASH("Nope");
+        return Slot(nullptr, nullptr);
+      });
+  if (!slot.IsLive()) {
+    // Initialize the slot, indicating that it's no longer free.
+    if (slot.IsRemoved()) {
       mRemovedCount--;
       keyHash |= kCollisionFlag;
     }
     if (mOps->initEntry) {
-      mOps->initEntry(entry, aKey);
+      mOps->initEntry(slot.ToEntry(), aKey);
     }
-    entry->mKeyHash = keyHash;
+    slot.SetKeyHash(keyHash);
     mEntryCount++;
   }
 
-  return entry;
+  return slot.ToEntry();
 }
 
-PLDHashEntryHdr*
-PLDHashTable::Add(const void* aKey)
-{
+PLDHashEntryHdr* PLDHashTable::Add(const void* aKey) {
   PLDHashEntryHdr* entry = Add(aKey, fallible);
   if (!entry) {
     if (!mEntryStore.Get()) {
       // We OOM'd while allocating the initial entry storage.
       uint32_t nbytes;
-      (void) SizeOfEntryStore(CapacityFromHashShift(), mEntrySize, &nbytes);
+      (void)SizeOfEntryStore(CapacityFromHashShift(), mEntrySize, &nbytes);
       NS_ABORT_OOM(nbytes);
     } else {
       // We failed to resize the existing entry storage, either due to OOM or
@@ -624,27 +582,29 @@ PLDHashTable::Add(const void* aKey)
   return entry;
 }
 
-void
-PLDHashTable::Remove(const void* aKey)
-{
-#ifdef DEBUG
+void PLDHashTable::Remove(const void* aKey) {
+#ifdef MOZ_HASH_TABLE_CHECKS_ENABLED
   AutoWriteOp op(mChecker);
 #endif
 
-  PLDHashEntryHdr* entry = mEntryStore.Get()
-                         ? SearchTable<ForSearchOrRemove>(aKey,
-                                                          ComputeKeyHash(aKey))
-                         : nullptr;
-  if (entry) {
-    RawRemove(entry);
-    ShrinkIfAppropriate();
+  if (!mEntryStore.Get()) {
+    return;
   }
+
+  PLDHashNumber keyHash = ComputeKeyHash(aKey);
+  SearchTable<ForSearchOrRemove>(
+      aKey, keyHash,
+      [&](Slot& slot) {
+        RawRemove(slot);
+        ShrinkIfAppropriate();
+      },
+      [&]() {
+        // Do nothing.
+      });
 }
 
-void
-PLDHashTable::RemoveEntry(PLDHashEntryHdr* aEntry)
-{
-#ifdef DEBUG
+void PLDHashTable::RemoveEntry(PLDHashEntryHdr* aEntry) {
+#ifdef MOZ_HASH_TABLE_CHECKS_ENABLED
   AutoWriteOp op(mChecker);
 #endif
 
@@ -652,9 +612,12 @@ PLDHashTable::RemoveEntry(PLDHashEntryHdr* aEntry)
   ShrinkIfAppropriate();
 }
 
-void
-PLDHashTable::RawRemove(PLDHashEntryHdr* aEntry)
-{
+void PLDHashTable::RawRemove(PLDHashEntryHdr* aEntry) {
+  Slot slot(mEntryStore.SlotForPLDHashEntry(aEntry, Capacity(), mEntrySize));
+  RawRemove(slot);
+}
+
+void PLDHashTable::RawRemove(Slot& aSlot) {
   // Unfortunately, we can only do weak checking here. That's because
   // RawRemove() can be called legitimately while an Enumerate() call is
   // active, which doesn't fit well into how Checker's mState variable works.
@@ -662,16 +625,17 @@ PLDHashTable::RawRemove(PLDHashEntryHdr* aEntry)
 
   MOZ_ASSERT(mEntryStore.Get());
 
-  MOZ_ASSERT(EntryIsLive(aEntry), "EntryIsLive(aEntry)");
+  MOZ_ASSERT(aSlot.IsLive());
 
   // Load keyHash first in case clearEntry() goofs it.
-  PLDHashNumber keyHash = aEntry->mKeyHash;
-  mOps->clearEntry(this, aEntry);
+  PLDHashEntryHdr* entry = aSlot.ToEntry();
+  PLDHashNumber keyHash = aSlot.KeyHash();
+  mOps->clearEntry(this, entry);
   if (keyHash & kCollisionFlag) {
-    MarkEntryRemoved(aEntry);
+    aSlot.MarkRemoved();
     mRemovedCount++;
   } else {
-    MarkEntryFree(aEntry);
+    aSlot.MarkFree();
   }
   mEntryCount--;
 }
@@ -679,67 +643,59 @@ PLDHashTable::RawRemove(PLDHashEntryHdr* aEntry)
 // Shrink or compress if a quarter or more of all entries are removed, or if the
 // table is underloaded according to the minimum alpha, and is not minimal-size
 // already.
-void
-PLDHashTable::ShrinkIfAppropriate()
-{
+void PLDHashTable::ShrinkIfAppropriate() {
   uint32_t capacity = Capacity();
   if (mRemovedCount >= capacity >> 2 ||
       (capacity > kMinCapacity && mEntryCount <= MinLoad(capacity))) {
     uint32_t log2;
     BestCapacity(mEntryCount, &capacity, &log2);
 
-    int32_t deltaLog2 = log2 - (kHashBits - mHashShift);
+    int32_t deltaLog2 = log2 - (kPLDHashNumberBits - mHashShift);
     MOZ_ASSERT(deltaLog2 <= 0);
 
-    (void) ChangeTable(deltaLog2);
+    (void)ChangeTable(deltaLog2);
   }
 }
 
-size_t
-PLDHashTable::ShallowSizeOfExcludingThis(MallocSizeOf aMallocSizeOf) const
-{
-#ifdef DEBUG
+size_t PLDHashTable::ShallowSizeOfExcludingThis(
+    MallocSizeOf aMallocSizeOf) const {
+#ifdef MOZ_HASH_TABLE_CHECKS_ENABLED
   AutoReadOp op(mChecker);
 #endif
 
   return aMallocSizeOf(mEntryStore.Get());
 }
 
-size_t
-PLDHashTable::ShallowSizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const
-{
+size_t PLDHashTable::ShallowSizeOfIncludingThis(
+    MallocSizeOf aMallocSizeOf) const {
   return aMallocSizeOf(this) + ShallowSizeOfExcludingThis(aMallocSizeOf);
 }
 
 PLDHashTable::Iterator::Iterator(Iterator&& aOther)
-  : mTable(aOther.mTable)
-  , mStart(aOther.mStart)
-  , mLimit(aOther.mLimit)
-  , mCurrent(aOther.mCurrent)
-  , mNexts(aOther.mNexts)
-  , mNextsLimit(aOther.mNextsLimit)
-  , mHaveRemoved(aOther.mHaveRemoved)
-{
+    : mTable(aOther.mTable),
+      mCurrent(aOther.mCurrent),
+      mNexts(aOther.mNexts),
+      mNextsLimit(aOther.mNextsLimit),
+      mHaveRemoved(aOther.mHaveRemoved),
+      mEntrySize(aOther.mEntrySize) {
   // No need to change |mChecker| here.
   aOther.mTable = nullptr;
-  aOther.mStart = nullptr;
-  aOther.mLimit = nullptr;
-  aOther.mCurrent = nullptr;
+  // We don't really have the concept of a null slot, so leave mCurrent.
   aOther.mNexts = 0;
   aOther.mNextsLimit = 0;
   aOther.mHaveRemoved = false;
+  aOther.mEntrySize = 0;
 }
 
 PLDHashTable::Iterator::Iterator(PLDHashTable* aTable)
-  : mTable(aTable)
-  , mStart(mTable->mEntryStore.Get())
-  , mLimit(mTable->mEntryStore.Get() + mTable->Capacity() * mTable->mEntrySize)
-  , mCurrent(mTable->mEntryStore.Get())
-  , mNexts(0)
-  , mNextsLimit(mTable->EntryCount())
-  , mHaveRemoved(false)
-{
-#ifdef DEBUG
+    : mTable(aTable),
+      mCurrent(mTable->mEntryStore.SlotForIndex(0, mTable->mEntrySize,
+                                                mTable->Capacity())),
+      mNexts(0),
+      mNextsLimit(mTable->EntryCount()),
+      mHaveRemoved(false),
+      mEntrySize(aTable->mEntrySize) {
+#ifdef MOZ_HASH_TABLE_CHECKS_ENABLED
   mTable->mChecker.StartReadOp();
 #endif
 
@@ -747,73 +703,110 @@ PLDHashTable::Iterator::Iterator(PLDHashTable* aTable)
       mTable->Capacity() > 0) {
     // Start iterating at a random entry. It would be even more chaotic to
     // iterate in fully random order, but that's harder.
-    mCurrent += ChaosMode::randomUint32LessThan(mTable->Capacity()) *
-                mTable->mEntrySize;
+    uint32_t capacity = mTable->CapacityFromHashShift();
+    uint32_t i = ChaosMode::randomUint32LessThan(capacity);
+    mCurrent =
+        mTable->mEntryStore.SlotForIndex(i, mTable->mEntrySize, capacity);
   }
 
   // Advance to the first live entry, if there is one.
-  if (!Done()) {
-    while (IsOnNonLiveEntry()) {
-      MoveToNextEntry();
-    }
+  if (!Done() && IsOnNonLiveEntry()) {
+    MoveToNextLiveEntry();
   }
 }
 
-PLDHashTable::Iterator::~Iterator()
-{
+PLDHashTable::Iterator::Iterator(PLDHashTable* aTable, EndIteratorTag aTag)
+    : mTable(aTable),
+      mCurrent(mTable->mEntryStore.SlotForIndex(0, mTable->mEntrySize,
+                                                mTable->Capacity())),
+      mNexts(mTable->EntryCount()),
+      mNextsLimit(mTable->EntryCount()),
+      mHaveRemoved(false),
+      mEntrySize(aTable->mEntrySize) {
+#ifdef MOZ_HASH_TABLE_CHECKS_ENABLED
+  mTable->mChecker.StartReadOp();
+#endif
+
+  MOZ_ASSERT(Done());
+}
+
+PLDHashTable::Iterator::Iterator(const Iterator& aOther)
+    : mTable(aOther.mTable),
+      mCurrent(aOther.mCurrent),
+      mNexts(aOther.mNexts),
+      mNextsLimit(aOther.mNextsLimit),
+      mHaveRemoved(aOther.mHaveRemoved),
+      mEntrySize(aOther.mEntrySize) {
+  // TODO: Is this necessary?
+  MOZ_ASSERT(!mHaveRemoved);
+
+#ifdef MOZ_HASH_TABLE_CHECKS_ENABLED
+  mTable->mChecker.StartReadOp();
+#endif
+}
+
+PLDHashTable::Iterator::~Iterator() {
   if (mTable) {
     if (mHaveRemoved) {
       mTable->ShrinkIfAppropriate();
     }
-#ifdef DEBUG
+#ifdef MOZ_HASH_TABLE_CHECKS_ENABLED
     mTable->mChecker.EndReadOp();
 #endif
   }
 }
 
-MOZ_ALWAYS_INLINE bool
-PLDHashTable::Iterator::IsOnNonLiveEntry() const
-{
+MOZ_ALWAYS_INLINE bool PLDHashTable::Iterator::IsOnNonLiveEntry() const {
   MOZ_ASSERT(!Done());
-  return !EntryIsLive(reinterpret_cast<PLDHashEntryHdr*>(mCurrent));
+  return !mCurrent.IsLive();
 }
 
-MOZ_ALWAYS_INLINE void
-PLDHashTable::Iterator::MoveToNextEntry()
-{
-  mCurrent += mTable->mEntrySize;
-  if (mCurrent == mLimit) {
-    mCurrent = mStart;  // Wrap-around. Possible due to Chaos Mode.
-  }
-}
-
-void
-PLDHashTable::Iterator::Next()
-{
+void PLDHashTable::Iterator::Next() {
   MOZ_ASSERT(!Done());
 
   mNexts++;
 
   // Advance to the next live entry, if there is one.
   if (!Done()) {
-    do {
-      MoveToNextEntry();
-    } while (IsOnNonLiveEntry());
+    MoveToNextLiveEntry();
   }
 }
 
-void
-PLDHashTable::Iterator::Remove()
-{
-  // This cast is needed for the same reason as the one in the destructor.
-  mTable->RawRemove(Get());
-  mHaveRemoved = true;
+MOZ_ALWAYS_INLINE void PLDHashTable::Iterator::MoveToNextLiveEntry() {
+  // Chaos mode requires wraparound to cover all possible entries, so we can't
+  // simply move to the next live entry and stop when we hit the end of the
+  // entry store. But we don't want to introduce extra branches into our inner
+  // loop. So we are going to exploit the structure of the entry store in this
+  // method to implement an efficient inner loop.
+  //
+  // The idea is that since we are really only iterating through the stored
+  // hashes and because we know that there are a power-of-two number of
+  // hashes, we can use masking to implement the wraparound for us. This
+  // method does have the downside of needing to recalculate where the
+  // associated entry is once we've found it, but that seems OK.
+
+  // Our current slot and its associated hash.
+  Slot slot = mCurrent;
+  PLDHashNumber* p = slot.HashPtr();
+  const uint32_t capacity = mTable->CapacityFromHashShift();
+  const uint32_t mask = capacity - 1;
+  auto hashes = reinterpret_cast<PLDHashNumber*>(mTable->mEntryStore.Get());
+  uint32_t slotIndex = p - hashes;
+
+  do {
+    slotIndex = (slotIndex + 1) & mask;
+  } while (!Slot::IsLiveHash(hashes[slotIndex]));
+
+  // slotIndex now indicates where a live slot is. Rematerialize the entry
+  // and the slot.
+  auto entries = reinterpret_cast<char*>(&hashes[capacity]);
+  char* entryPtr = entries + slotIndex * mEntrySize;
+  auto entry = reinterpret_cast<PLDHashEntryHdr*>(entryPtr);
+
+  mCurrent = Slot(entry, &hashes[slotIndex]);
 }
 
-#ifdef DEBUG
-void
-PLDHashTable::MarkImmutable()
-{
-  mChecker.SetNonWritable();
+void PLDHashTable::Iterator::Remove() {
+  mTable->RawRemove(mCurrent);
+  mHaveRemoved = true;
 }
-#endif
