@@ -17,7 +17,6 @@
 
 #include "nsAtom.h"
 #include "nsAtomTable.h"
-#include "nsAutoPtr.h"
 #include "nsCRT.h"
 #include "nsDataHashtable.h"
 #include "nsGkAtoms.h"
@@ -65,8 +64,7 @@ enum class GCKind {
 // This atomic can be accessed during the GC and other places where recorded
 // events are not allowed, so its value is not preserved when recording or
 // replaying.
-Atomic<int32_t, ReleaseAcquire, recordreplay::Behavior::DontPreserve>
-    nsDynamicAtom::gUnusedAtomCount;
+Atomic<int32_t, ReleaseAcquire> nsDynamicAtom::gUnusedAtomCount;
 
 nsDynamicAtom::nsDynamicAtom(const nsAString& aString, uint32_t aHash,
                              bool aIsAsciiLowercase)
@@ -627,7 +625,7 @@ already_AddRefed<nsAtom> nsAtomTable::AtomizeMainThread(
     RefPtr<nsAtom> newAtom =
         dont_AddRef(nsDynamicAtom::Create(aUTF16String, key.mHash));
     he->mAtom = newAtom;
-    retVal = newAtom.forget();
+    retVal = std::move(newAtom);
   }
 
   p.Set(retVal);
