@@ -9,7 +9,6 @@
 
 #include "nsBaseHashtable.h"
 #include "nsHashKeys.h"
-#include "nsAutoPtr.h"
 
 /**
  * templated hashtable class maps keys to reference pointers.
@@ -27,7 +26,7 @@ class nsRefPtrHashtable
   typedef PtrType* UserDataType;
   typedef nsBaseHashtable<KeyClass, RefPtr<PtrType>, PtrType*> base_type;
 
-  nsRefPtrHashtable() {}
+  nsRefPtrHashtable() = default;
   explicit nsRefPtrHashtable(uint32_t aInitLength)
       : nsBaseHashtable<KeyClass, RefPtr<PtrType>, PtrType*>(aInitLength) {}
 
@@ -51,13 +50,17 @@ class nsRefPtrHashtable
    */
   PtrType* GetWeak(KeyType aKey, bool* aFound = nullptr) const;
 
-  // Overload Put, rather than overriding it.
-  using base_type::Put;
+  // Hide base class' Put overloads intentionally, to make any necessary
+  // refcounting explicit when calling Put.
 
-  void Put(KeyType aKey, already_AddRefed<PtrType> aData);
+  template <typename U,
+            typename = std::enable_if_t<std::is_base_of_v<PtrType, U>>>
+  void Put(KeyType aKey, RefPtr<U>&& aData);
 
-  MOZ_MUST_USE bool Put(KeyType aKey, already_AddRefed<PtrType> aData,
-                        const mozilla::fallible_t&);
+  template <typename U,
+            typename = std::enable_if_t<std::is_base_of_v<PtrType, U>>>
+  [[nodiscard]] bool Put(KeyType aKey, RefPtr<U>&& aData,
+                         const mozilla::fallible_t&);
 
   /**
    * Remove the entry associated with aKey (if any), optionally _moving_ its
@@ -96,7 +99,7 @@ bool nsRefPtrHashtable<KeyClass, PtrType>::Get(KeyType aKey,
 
   if (ent) {
     if (aRefPtr) {
-      *aRefPtr = ent->mData;
+      *aRefPtr = ent->GetData();
 
       NS_IF_ADDREF(*aRefPtr);
     }
@@ -121,7 +124,7 @@ already_AddRefed<PtrType> nsRefPtrHashtable<KeyClass, PtrType>::Get(
     return nullptr;
   }
 
-  RefPtr<PtrType> copy = ent->mData;
+  RefPtr<PtrType> copy = ent->GetData();
   return copy.forget();
 }
 
@@ -135,7 +138,7 @@ PtrType* nsRefPtrHashtable<KeyClass, PtrType>::GetWeak(KeyType aKey,
       *aFound = true;
     }
 
-    return ent->mData;
+    return ent->GetData();
   }
 
   // Key does not exist, return nullptr and set aFound to false
@@ -147,16 +150,17 @@ PtrType* nsRefPtrHashtable<KeyClass, PtrType>::GetWeak(KeyType aKey,
 }
 
 template <class KeyClass, class PtrType>
-void nsRefPtrHashtable<KeyClass, PtrType>::Put(
-    KeyType aKey, already_AddRefed<PtrType> aData) {
+template <typename U, typename>
+void nsRefPtrHashtable<KeyClass, PtrType>::Put(KeyType aKey,
+                                               RefPtr<U>&& aData) {
   if (!Put(aKey, std::move(aData), mozilla::fallible)) {
     NS_ABORT_OOM(this->mTable.EntrySize() * this->mTable.EntryCount());
   }
 }
 
 template <class KeyClass, class PtrType>
-bool nsRefPtrHashtable<KeyClass, PtrType>::Put(KeyType aKey,
-                                               already_AddRefed<PtrType> aData,
+template <typename U, typename>
+bool nsRefPtrHashtable<KeyClass, PtrType>::Put(KeyType aKey, RefPtr<U>&& aData,
                                                const mozilla::fallible_t&) {
   typename base_type::EntryType* ent = this->PutEntry(aKey, mozilla::fallible);
 
@@ -164,7 +168,7 @@ bool nsRefPtrHashtable<KeyClass, PtrType>::Put(KeyType aKey,
     return false;
   }
 
-  ent->mData = aData;
+  ent->SetData(std::move(aData));
 
   return true;
 }
@@ -176,7 +180,7 @@ bool nsRefPtrHashtable<KeyClass, PtrType>::Remove(KeyType aKey,
 
   if (ent) {
     if (aRefPtr) {
-      ent->mData.forget(aRefPtr);
+      ent->GetModifiableData()->forget(aRefPtr);
     }
     this->RemoveEntry(ent);
     return true;
