@@ -31,7 +31,7 @@ static nsresult String2Double(const char* aString, double* aResult) {
 }
 
 static nsresult AString2Double(const nsAString& aString, double* aResult) {
-  char* pChars = ToNewCString(aString);
+  char* pChars = ToNewCString(aString, mozilla::fallible);
   if (!pChars) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -331,7 +331,7 @@ static nsresult CloneArray(uint16_t aInType, const nsIID* aInIID,
       if (aOutIID) {
         *aOutIID = *aInIID;
       }
-      MOZ_FALLTHROUGH;
+      [[fallthrough]];
 
     case nsIDataType::VTYPE_INTERFACE: {
       memcpy(*aOutValue, aInValue, allocSize);
@@ -633,7 +633,7 @@ bool nsDiscriminatedUnion::String2ID(nsID* aPid) const {
       return false;
   }
 
-  char* pChars = ToNewCString(*pString);
+  char* pChars = ToNewCString(*pString, mozilla::fallible);
   if (!pChars) {
     return false;
   }
@@ -688,7 +688,7 @@ nsresult nsDiscriminatedUnion::ToString(nsACString& aOutString) const {
     case nsIDataType::VTYPE_WSTRING_SIZE_IS:
     case nsIDataType::VTYPE_WCHAR:
       NS_ERROR("ToString being called for a string type - screwy logic!");
-      MOZ_FALLTHROUGH;
+      [[fallthrough]];
 
       // XXX We might want stringified versions of these... ???
 
@@ -1161,9 +1161,7 @@ nsresult nsDiscriminatedUnion::SetFromVariant(nsIVariant* aValue) {
     case nsIDataType::VTYPE_WSTRING_SIZE_IS:
       CASE__SET_FROM_VARIANT_VTYPE_PROLOGUE(VTYPE_ASTRING);
       u.mAStringValue = new nsString();
-      if (!u.mAStringValue) {
-        return NS_ERROR_OUT_OF_MEMORY;
-      }
+
       rv = aValue->GetAsAString(*u.mAStringValue);
       if (NS_FAILED(rv)) {
         delete u.mAStringValue;
@@ -1173,9 +1171,7 @@ nsresult nsDiscriminatedUnion::SetFromVariant(nsIVariant* aValue) {
     case nsIDataType::VTYPE_CSTRING:
       CASE__SET_FROM_VARIANT_VTYPE_PROLOGUE(VTYPE_CSTRING);
       u.mCStringValue = new nsCString();
-      if (!u.mCStringValue) {
-        return NS_ERROR_OUT_OF_MEMORY;
-      }
+
       rv = aValue->GetAsACString(*u.mCStringValue);
       if (NS_FAILED(rv)) {
         delete u.mCStringValue;
@@ -1185,9 +1181,7 @@ nsresult nsDiscriminatedUnion::SetFromVariant(nsIVariant* aValue) {
     case nsIDataType::VTYPE_UTF8STRING:
       CASE__SET_FROM_VARIANT_VTYPE_PROLOGUE(VTYPE_UTF8STRING);
       u.mUTF8StringValue = new nsUTF8String();
-      if (!u.mUTF8StringValue) {
-        return NS_ERROR_OUT_OF_MEMORY;
-      }
+
       rv = aValue->GetAsAUTF8String(*u.mUTF8StringValue);
       if (NS_FAILED(rv)) {
         delete u.mUTF8StringValue;
