@@ -12,7 +12,6 @@
 #include "nsString.h"
 #include "nsCRT.h"
 #include "nsIFile.h"
-#include "nsIFactory.h"
 #include "nsILocalFileWin.h"
 #include "nsIClassInfoImpl.h"
 #include "prio.h"
@@ -47,6 +46,8 @@ class nsLocalFile final : public nsILocalFileWin {
   // Removes registry command handler parameters, quotes, and expands
   // environment strings.
   static bool CleanupCmdHandlerPath(nsAString& aCommandHandler);
+  // Called off the main thread to open the window revealing the file
+  static nsresult RevealFile(const nsString& aResolvedPath);
 
  private:
   // CopyMove and CopySingleFile constants for |options| parameter:
@@ -62,7 +63,8 @@ class nsLocalFile final : public nsILocalFileWin {
 
   bool mDirty;  // cached information can only be used when this is false
   bool mResolveDirty;
-  bool mFollowSymlinks;  // should we follow symlinks when working on this file
+
+  bool mUseDOSDevicePathSyntax;
 
   // this string will always be in native format!
   nsString mWorkingPath;
@@ -83,9 +85,11 @@ class nsLocalFile final : public nsILocalFileWin {
     mShortWorkingPath.Truncate();
   }
 
+  nsresult LookupExtensionIn(const char* const* aExtensionsArray,
+                             size_t aArrayLength, bool* aResult);
+
   nsresult ResolveAndStat();
   nsresult Resolve();
-  nsresult ResolveShortcut();
 
   void EnsureShortPath();
 
