@@ -324,6 +324,7 @@ typename nsTSubstring<T>::size_type nsTSubstring<T>::Capacity() const {
       capacity = (hdr->StorageSize() / sizeof(char_type)) - 1;
     }
   } else if (this->mDataFlags & DataFlags::INLINE) {
+    MOZ_ASSERT(this->mClassFlags & ClassFlags::INLINE);
     capacity = AsAutoString(this)->mInlineCapacity;
   } else if (this->mDataFlags & DataFlags::OWNED) {
     // we don't store the capacity of an adopted buffer because that would
@@ -656,45 +657,6 @@ bool nsTSubstring<T>::Replace(index_type aCutStart, size_type aCutLength,
   return true;
 }
 
-template <typename T>
-void nsTSubstring<T>::ReplaceASCII(index_type aCutStart, size_type aCutLength,
-                                   const char* aData, size_type aLength) {
-  if (!ReplaceASCII(aCutStart, aCutLength, aData, aLength, mozilla::fallible)) {
-    AllocFailed(this->Length() - aCutLength + 1);
-  }
-}
-
-template <typename T>
-bool nsTSubstring<T>::ReplaceASCII(index_type aCutStart, size_type aCutLength,
-                                   const char* aData, size_type aLength,
-                                   const fallible_t& aFallible) {
-  if (aLength == size_type(-1)) {
-    aLength = strlen(aData);
-  }
-
-  // A Unicode string can't depend on an ASCII string buffer,
-  // so this dependence check only applies to CStrings.
-#ifdef CharT_is_char
-  if (this->IsDependentOn(aData, aData + aLength)) {
-    nsTAutoString_CharT temp(aData, aLength);
-    return Replace(aCutStart, aCutLength, temp, aFallible);
-  }
-#endif
-
-  aCutStart = XPCOM_MIN(aCutStart, this->Length());
-
-  bool ok = ReplacePrep(aCutStart, aCutLength, aLength);
-  if (!ok) {
-    return false;
-  }
-
-  if (aLength > 0) {
-    char_traits::copyASCII(this->mData + aCutStart, aData, aLength);
-  }
-
-  return true;
-}
-
 template <typename T>
 void nsTSubstring<T>::Replace(index_type aCutStart, size_type aCutLength,
                               const substring_tuple_type& aTuple) {
@@ -996,7 +958,7 @@ bool nsTStringRepr<T>::Equals(const self_type& aStr) const {
 
 template <typename T>
 bool nsTStringRepr<T>::Equals(const self_type& aStr,
-                              const comparator_type& aComp) const {
+                              comparator_type aComp) const {
   return this->mLength == aStr.mLength &&
          aComp(this->mData, aStr.mData, this->mLength, aStr.mLength) == 0;
 }
@@ -1008,7 +970,7 @@ bool nsTStringRepr<T>::Equals(const substring_tuple_type& aTuple) const {
 
 template <typename T>
 bool nsTStringRepr<T>::Equals(const substring_tuple_type& aTuple,
-                              const comparator_type& aComp) const {
+                              comparator_type aComp) const {
   return Equals(substring_type(aTuple), aComp);
 }
 
@@ -1028,7 +990,7 @@ bool nsTStringRepr<T>::Equals(const char_type* aData) const {
 
 template <typename T>
 bool nsTStringRepr<T>::Equals(const char_type* aData,
-                              const comparator_type& aComp) const {
+                              comparator_type aComp) const {
   // unfortunately, some callers pass null :-(
   if (!aData) {
     MOZ_ASSERT_UNREACHABLE("null data pointer");
@@ -1053,6 +1015,13 @@ bool nsTStringRepr<T>::EqualsASCII(const char* aData) const {
                                                  aData) == 0;
 }
 
+template <typename T>
+bool nsTStringRepr<T>::EqualsLatin1(const char* aData,
+                                    const size_type aLength) const {
+  return (this->mLength == aLength) &&
+         char_traits::equalsLatin1(this->mData, aData, aLength);
+}
+
 template <typename T>
 bool nsTStringRepr<T>::LowerCaseEqualsASCII(const char* aData,
                                             size_type aLen) const {
@@ -1087,6 +1056,11 @@ int32_t nsTStringRepr<T>::FindChar(char_type aChar, index_type aOffset) const {
   return -1;
 }
 
+template <typename T>
+bool nsTStringRepr<T>::Contains(char_type aChar) const {
+  return FindChar(aChar) != kNotFound;
+}
+
 }  // namespace detail
 }  // namespace mozilla
 
