@@ -11,7 +11,6 @@
 #include <stdio.h>
 #include <iostream>
 #include "nsTArray.h"
-#include "nsAutoPtr.h"
 #include "nsString.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsDirectoryServiceUtils.h"
@@ -36,7 +35,7 @@ inline bool operator<(const nsCOMPtr<T>& lhs, const nsCOMPtr<T>& rhs) {
 template <class ElementType>
 static bool test_basic_array(ElementType* data, size_t dataLen,
                              const ElementType& extra) {
-  nsTArray<ElementType> ary;
+  CopyableTArray<ElementType> ary;
   const nsTArray<ElementType>& cary = ary;
 
   ary.AppendElements(data, dataLen);
@@ -121,7 +120,7 @@ static bool test_basic_array(ElementType* data, size_t dataLen,
           return false;
 
   size_t index = ary.Length() / 2;
-  if (!ary.InsertElementAt(index, extra)) return false;
+  ary.InsertElementAt(index, extra);
   if (!(ary == ary)) return false;
   if (ary[index] != extra) return false;
   if (ary.IndexOf(extra) == ary.NoIndex) return false;
@@ -144,12 +143,12 @@ static bool test_basic_array(ElementType* data, size_t dataLen,
           []() { return false; }))
     return false;
 
-  nsTArray<ElementType> copy(ary);
+  nsTArray<ElementType> copy(ary.Clone());
   if (!(ary == copy)) return false;
   for (i = 0; i < copy.Length(); ++i) {
     if (ary[i] != copy[i]) return false;
   }
-  if (!ary.AppendElements(copy)) return false;
+  ary.AppendElements(copy);
   size_t cap = ary.Capacity();
   ary.RemoveElementsAt(copy.Length(), copy.Length());
   ary.Compact();
@@ -166,7 +165,7 @@ static bool test_basic_array(ElementType* data, size_t dataLen,
     return false;
 
   ary.Clear();
-  if (!ary.IsEmpty() || ary.Elements() == nullptr) return false;
+  if (!ary.IsEmpty()) return false;
   if (!(ary == nsTArray<ElementType>())) return false;
   if (ary == copy) return false;
   if (ary.SafeElementAt(0, extra) != extra ||
@@ -179,7 +178,7 @@ static bool test_basic_array(ElementType* data, size_t dataLen,
     if (ary[i] != copy[i]) return false;
   }
 
-  if (!ary.InsertElementsAt(0, copy)) return false;
+  ary.InsertElementsAt(0, copy);
   if (ary == copy) return false;
   ary.RemoveElementsAt(0, copy.Length());
   for (i = 0; i < copy.Length(); ++i) {
@@ -228,14 +227,10 @@ class Object {
  public:
   Object() : mNum(0) {}
   Object(const char* str, uint32_t num) : mStr(str), mNum(num) {}
-  Object(const Object& other) : mStr(other.mStr), mNum(other.mNum) {}
-  ~Object() {}
+  Object(const Object& other) = default;
+  ~Object() = default;
 
-  Object& operator=(const Object& other) {
-    mStr = other.mStr;
-    mNum = other.mNum;
-    return *this;
-  }
+  Object& operator=(const Object& other) = default;
 
   bool operator==(const Object& other) const {
     return mStr == other.mStr && mNum == other.mNum;
@@ -261,7 +256,7 @@ TEST(TArray, test_object_array)
   size_t i;
   for (i = 0; i < ArrayLength(kdata); ++i) {
     char x[] = {kdata[i], '\0'};
-    ASSERT_TRUE(objArray.AppendElement(Object(x, i)));
+    objArray.AppendElement(Object(x, i));
   }
   for (i = 0; i < ArrayLength(kdata); ++i) {
     ASSERT_EQ(objArray[i].Str()[0], kdata[i]);
@@ -300,6 +295,61 @@ class Moveable {
   static int Count() { return sCount; }
 };
 
+class MoveOnly_RelocateUsingMemutils {
+ public:
+  MoveOnly_RelocateUsingMemutils() = default;
+
+  MoveOnly_RelocateUsingMemutils(const MoveOnly_RelocateUsingMemutils&) =
+      delete;
+  MoveOnly_RelocateUsingMemutils(MoveOnly_RelocateUsingMemutils&&) = default;
+
+  MoveOnly_RelocateUsingMemutils& operator=(
+      const MoveOnly_RelocateUsingMemutils&) = delete;
+  MoveOnly_RelocateUsingMemutils& operator=(MoveOnly_RelocateUsingMemutils&&) =
+      default;
+};
+
+static_assert(
+    std::is_move_constructible_v<nsTArray<MoveOnly_RelocateUsingMemutils>>);
+static_assert(
+    std::is_move_assignable_v<nsTArray<MoveOnly_RelocateUsingMemutils>>);
+static_assert(
+    !std::is_copy_constructible_v<nsTArray<MoveOnly_RelocateUsingMemutils>>);
+static_assert(
+    !std::is_copy_assignable_v<nsTArray<MoveOnly_RelocateUsingMemutils>>);
+
+class MoveOnly_RelocateUsingMoveConstructor {
+ public:
+  MoveOnly_RelocateUsingMoveConstructor() = default;
+
+  MoveOnly_RelocateUsingMoveConstructor(
+      const MoveOnly_RelocateUsingMoveConstructor&) = delete;
+  MoveOnly_RelocateUsingMoveConstructor(
+      MoveOnly_RelocateUsingMoveConstructor&&) = default;
+
+  MoveOnly_RelocateUsingMoveConstructor& operator=(
+      const MoveOnly_RelocateUsingMoveConstructor&) = delete;
+  MoveOnly_RelocateUsingMoveConstructor& operator=(
+      MoveOnly_RelocateUsingMoveConstructor&&) = default;
+};
+}  // namespace TestTArray
+
+MOZ_DECLARE_RELOCATE_USING_MOVE_CONSTRUCTOR(
+    TestTArray::MoveOnly_RelocateUsingMoveConstructor)
+
+namespace TestTArray {
+static_assert(std::is_move_constructible_v<
+              nsTArray<MoveOnly_RelocateUsingMoveConstructor>>);
+static_assert(
+    std::is_move_assignable_v<nsTArray<MoveOnly_RelocateUsingMoveConstructor>>);
+static_assert(!std::is_copy_constructible_v<
+              nsTArray<MoveOnly_RelocateUsingMoveConstructor>>);
+static_assert(!std::is_copy_assignable_v<
+              nsTArray<MoveOnly_RelocateUsingMoveConstructor>>);
+}  // namespace TestTArray
+
+namespace TestTArray {
+
 /* static */
 int Countable::sCount = 0;
 /* static */
@@ -321,7 +371,7 @@ TEST(TArray, test_move_array)
   nsTArray<Countable> countableArray;
   uint32_t i;
   for (i = 0; i < 4; ++i) {
-    ASSERT_TRUE(countableArray.AppendElement(Countable()));
+    countableArray.AppendElement(Countable());
   }
 
   ASSERT_EQ(Countable::Count(), 8);
@@ -330,7 +380,7 @@ TEST(TArray, test_move_array)
 
   ASSERT_EQ(Countable::Count(), 8);
 
-  nsTArray<Countable> copyCountableArray(constRefCountableArray);
+  nsTArray<Countable> copyCountableArray(constRefCountableArray.Clone());
 
   ASSERT_EQ(Countable::Count(), 12);
 
@@ -367,7 +417,7 @@ TEST(TArray, test_move_array)
 
   nsTArray<Moveable> moveableArray;
   for (i = 0; i < 4; ++i) {
-    ASSERT_TRUE(moveableArray.AppendElement(Moveable()));
+    moveableArray.AppendElement(Moveable());
   }
 
   ASSERT_EQ(Moveable::Count(), 4);
@@ -376,7 +426,7 @@ TEST(TArray, test_move_array)
 
   ASSERT_EQ(Moveable::Count(), 4);
 
-  nsTArray<Moveable> copyMoveableArray(constRefMoveableArray);
+  nsTArray<Moveable> copyMoveableArray(constRefMoveableArray.Clone());
 
   ASSERT_EQ(Moveable::Count(), 8);
 
@@ -413,7 +463,7 @@ TEST(TArray, test_move_array)
 
   AutoTArray<Moveable, 8> moveableAutoArray;
   for (uint32_t i = 0; i < 4; ++i) {
-    ASSERT_TRUE(moveableAutoArray.AppendElement(Moveable()));
+    moveableAutoArray.AppendElement(Moveable());
   }
 
   ASSERT_EQ(Moveable::Count(), 12);
@@ -422,7 +472,8 @@ TEST(TArray, test_move_array)
 
   ASSERT_EQ(Moveable::Count(), 12);
 
-  AutoTArray<Moveable, 8> copyMoveableAutoArray(constRefMoveableAutoArray);
+  CopyableAutoTArray<Moveable, 8> copyMoveableAutoArray(
+      constRefMoveableAutoArray);
 
   ASSERT_EQ(Moveable::Count(), 16);
 
@@ -431,6 +482,183 @@ TEST(TArray, test_move_array)
   ASSERT_EQ(Moveable::Count(), 16);
 }
 
+template <typename TypeParam>
+class TArray_MoveOnlyTest : public ::testing::Test {};
+
+TYPED_TEST_CASE_P(TArray_MoveOnlyTest);
+
+static constexpr size_t kMoveOnlyTestArrayLength = 4;
+
+template <typename ArrayType>
+static auto MakeMoveOnlyArray() {
+  ArrayType moveOnlyArray;
+  for (size_t i = 0; i < kMoveOnlyTestArrayLength; ++i) {
+    EXPECT_TRUE(
+        moveOnlyArray.AppendElement(typename ArrayType::elem_type(), fallible));
+  }
+  return moveOnlyArray;
+}
+
+TYPED_TEST_P(TArray_MoveOnlyTest, nsTArray_MoveConstruct) {
+  auto moveOnlyArray = MakeMoveOnlyArray<nsTArray<TypeParam>>();
+  nsTArray<TypeParam> movedMoveOnlyArray(std::move(moveOnlyArray));
+
+  ASSERT_EQ(0u, moveOnlyArray.Length());
+  ASSERT_EQ(kMoveOnlyTestArrayLength, movedMoveOnlyArray.Length());
+}
+
+TYPED_TEST_P(TArray_MoveOnlyTest, nsTArray_MoveAssign) {
+  auto moveOnlyArray = MakeMoveOnlyArray<nsTArray<TypeParam>>();
+  nsTArray<TypeParam> movedMoveOnlyArray;
+  movedMoveOnlyArray = std::move(moveOnlyArray);
+
+  ASSERT_EQ(0u, moveOnlyArray.Length());
+  ASSERT_EQ(kMoveOnlyTestArrayLength, movedMoveOnlyArray.Length());
+}
+
+TYPED_TEST_P(TArray_MoveOnlyTest, nsTArray_MoveReAssign) {
+  nsTArray<TypeParam> movedMoveOnlyArray;
+  movedMoveOnlyArray = MakeMoveOnlyArray<nsTArray<TypeParam>>();
+  // Re-assign, to check that move-assign does not only work on an empty array.
+  movedMoveOnlyArray = MakeMoveOnlyArray<nsTArray<TypeParam>>();
+
+  ASSERT_EQ(kMoveOnlyTestArrayLength, movedMoveOnlyArray.Length());
+}
+
+TYPED_TEST_P(TArray_MoveOnlyTest, nsTArray_to_FallibleTArray_MoveConstruct) {
+  auto moveOnlyArray = MakeMoveOnlyArray<nsTArray<TypeParam>>();
+  FallibleTArray<TypeParam> differentAllocatorMoveOnlyArray(
+      std::move(moveOnlyArray));
+
+  ASSERT_EQ(0u, moveOnlyArray.Length());
+  ASSERT_EQ(kMoveOnlyTestArrayLength, differentAllocatorMoveOnlyArray.Length());
+}
+
+TYPED_TEST_P(TArray_MoveOnlyTest, nsTArray_to_FallibleTArray_MoveAssign) {
+  auto moveOnlyArray = MakeMoveOnlyArray<nsTArray<TypeParam>>();
+  FallibleTArray<TypeParam> differentAllocatorMoveOnlyArray;
+  differentAllocatorMoveOnlyArray = std::move(moveOnlyArray);
+
+  ASSERT_EQ(0u, moveOnlyArray.Length());
+  ASSERT_EQ(kMoveOnlyTestArrayLength, differentAllocatorMoveOnlyArray.Length());
+}
+
+TYPED_TEST_P(TArray_MoveOnlyTest, FallibleTArray_to_nsTArray_MoveConstruct) {
+  auto moveOnlyArray = MakeMoveOnlyArray<FallibleTArray<TypeParam>>();
+  nsTArray<TypeParam> differentAllocatorMoveOnlyArray(std::move(moveOnlyArray));
+
+  ASSERT_EQ(0u, moveOnlyArray.Length());
+  ASSERT_EQ(kMoveOnlyTestArrayLength, differentAllocatorMoveOnlyArray.Length());
+}
+
+TYPED_TEST_P(TArray_MoveOnlyTest, FallibleTArray_to_nsTArray_MoveAssign) {
+  auto moveOnlyArray = MakeMoveOnlyArray<FallibleTArray<TypeParam>>();
+  nsTArray<TypeParam> differentAllocatorMoveOnlyArray;
+  differentAllocatorMoveOnlyArray = std::move(moveOnlyArray);
+
+  ASSERT_EQ(0u, moveOnlyArray.Length());
+  ASSERT_EQ(kMoveOnlyTestArrayLength, differentAllocatorMoveOnlyArray.Length());
+}
+
+TYPED_TEST_P(TArray_MoveOnlyTest, AutoTArray_AutoStorage_MoveConstruct) {
+  auto moveOnlyArray =
+      MakeMoveOnlyArray<AutoTArray<TypeParam, kMoveOnlyTestArrayLength>>();
+  AutoTArray<TypeParam, kMoveOnlyTestArrayLength> autoMoveOnlyArray(
+      std::move(moveOnlyArray));
+
+  ASSERT_EQ(0u, moveOnlyArray.Length());
+  ASSERT_EQ(kMoveOnlyTestArrayLength, autoMoveOnlyArray.Length());
+}
+
+TYPED_TEST_P(TArray_MoveOnlyTest, AutoTArray_AutoStorage_MoveAssign) {
+  auto moveOnlyArray =
+      MakeMoveOnlyArray<AutoTArray<TypeParam, kMoveOnlyTestArrayLength>>();
+  AutoTArray<TypeParam, kMoveOnlyTestArrayLength> autoMoveOnlyArray;
+  autoMoveOnlyArray = std::move(moveOnlyArray);
+
+  ASSERT_EQ(0u, moveOnlyArray.Length());
+  ASSERT_EQ(kMoveOnlyTestArrayLength, autoMoveOnlyArray.Length());
+}
+
+TYPED_TEST_P(TArray_MoveOnlyTest,
+             nsTArray_to_AutoTArray_AutoStorage_MoveConstruct) {
+  auto moveOnlyArray = MakeMoveOnlyArray<nsTArray<TypeParam>>();
+  AutoTArray<TypeParam, kMoveOnlyTestArrayLength> autoMoveOnlyArray(
+      std::move(moveOnlyArray));
+
+  ASSERT_EQ(0u, moveOnlyArray.Length());
+  ASSERT_EQ(kMoveOnlyTestArrayLength, autoMoveOnlyArray.Length());
+}
+
+TYPED_TEST_P(TArray_MoveOnlyTest,
+             nsTArray_to_AutoTArray_AutoStorage_MoveAssign) {
+  auto moveOnlyArray = MakeMoveOnlyArray<nsTArray<TypeParam>>();
+  AutoTArray<TypeParam, kMoveOnlyTestArrayLength> autoMoveOnlyArray;
+  autoMoveOnlyArray = std::move(moveOnlyArray);
+
+  ASSERT_EQ(0u, moveOnlyArray.Length());
+  ASSERT_EQ(kMoveOnlyTestArrayLength, autoMoveOnlyArray.Length());
+}
+
+TYPED_TEST_P(TArray_MoveOnlyTest,
+             nsTArray_to_AutoTArray_HeapStorage_MoveConstruct) {
+  auto moveOnlyArray = MakeMoveOnlyArray<nsTArray<TypeParam>>();
+  AutoTArray<TypeParam, kMoveOnlyTestArrayLength - 1> autoMoveOnlyArray(
+      std::move(moveOnlyArray));
+
+  ASSERT_EQ(0u, moveOnlyArray.Length());
+  ASSERT_EQ(kMoveOnlyTestArrayLength, autoMoveOnlyArray.Length());
+}
+
+TYPED_TEST_P(TArray_MoveOnlyTest,
+             nsTArray_to_AutoTArray_HeapStorage_MoveAssign) {
+  auto moveOnlyArray = MakeMoveOnlyArray<nsTArray<TypeParam>>();
+  AutoTArray<TypeParam, kMoveOnlyTestArrayLength - 1> autoMoveOnlyArray;
+  autoMoveOnlyArray = std::move(moveOnlyArray);
+
+  ASSERT_EQ(0u, moveOnlyArray.Length());
+  ASSERT_EQ(kMoveOnlyTestArrayLength, autoMoveOnlyArray.Length());
+}
+
+TYPED_TEST_P(TArray_MoveOnlyTest,
+             FallibleTArray_to_AutoTArray_HeapStorage_MoveConstruct) {
+  auto moveOnlyArray = MakeMoveOnlyArray<FallibleTArray<TypeParam>>();
+  AutoTArray<TypeParam, 4> autoMoveOnlyArray(std::move(moveOnlyArray));
+
+  ASSERT_EQ(0u, moveOnlyArray.Length());
+  ASSERT_EQ(kMoveOnlyTestArrayLength, autoMoveOnlyArray.Length());
+}
+
+TYPED_TEST_P(TArray_MoveOnlyTest,
+             FallibleTArray_to_AutoTArray_HeapStorage_MoveAssign) {
+  auto moveOnlyArray = MakeMoveOnlyArray<FallibleTArray<TypeParam>>();
+  AutoTArray<TypeParam, 4> autoMoveOnlyArray;
+  autoMoveOnlyArray = std::move(moveOnlyArray);
+
+  ASSERT_EQ(0u, moveOnlyArray.Length());
+  ASSERT_EQ(kMoveOnlyTestArrayLength, autoMoveOnlyArray.Length());
+}
+
+REGISTER_TYPED_TEST_CASE_P(
+    TArray_MoveOnlyTest, nsTArray_MoveConstruct, nsTArray_MoveAssign,
+    nsTArray_MoveReAssign, nsTArray_to_FallibleTArray_MoveConstruct,
+    nsTArray_to_FallibleTArray_MoveAssign,
+    FallibleTArray_to_nsTArray_MoveConstruct,
+    FallibleTArray_to_nsTArray_MoveAssign, AutoTArray_AutoStorage_MoveConstruct,
+    AutoTArray_AutoStorage_MoveAssign,
+    nsTArray_to_AutoTArray_AutoStorage_MoveConstruct,
+    nsTArray_to_AutoTArray_AutoStorage_MoveAssign,
+    nsTArray_to_AutoTArray_HeapStorage_MoveConstruct,
+    nsTArray_to_AutoTArray_HeapStorage_MoveAssign,
+    FallibleTArray_to_AutoTArray_HeapStorage_MoveConstruct,
+    FallibleTArray_to_AutoTArray_HeapStorage_MoveAssign);
+
+using BothMoveOnlyTypes =
+    ::testing::Types<MoveOnly_RelocateUsingMemutils,
+                     MoveOnly_RelocateUsingMoveConstructor>;
+INSTANTIATE_TYPED_TEST_CASE_P(InstantiationOf, TArray_MoveOnlyTest,
+                              BothMoveOnlyTypes);
+
 //----
 
 TEST(TArray, test_string_array)
@@ -441,7 +669,7 @@ TEST(TArray, test_string_array)
   for (i = 0; i < ArrayLength(kdata); ++i) {
     nsCString str;
     str.Assign(kdata[i]);
-    ASSERT_TRUE(strArray.AppendElement(str));
+    strArray.AppendElement(str);
   }
   for (i = 0; i < ArrayLength(kdata); ++i) {
     ASSERT_EQ(strArray[i].CharAt(0), kdata[i]);
@@ -449,7 +677,7 @@ TEST(TArray, test_string_array)
 
   const char kextra[] = "foo bar";
   size_t oldLen = strArray.Length();
-  ASSERT_TRUE(strArray.AppendElement(kextra));
+  strArray.AppendElement(kextra);
   strArray.RemoveElement(kextra);
   ASSERT_EQ(oldLen, strArray.Length());
 
@@ -526,7 +754,7 @@ class RefcountedObject {
   void Release() {
     if (--rc == 0) delete this;
   }
-  ~RefcountedObject() {}
+  ~RefcountedObject() = default;
 
  private:
   int32_t rc;
@@ -534,7 +762,7 @@ class RefcountedObject {
 
 TEST(TArray, test_refptr_array)
 {
-  nsTArray<RefPtr<RefcountedObject> > objArray;
+  nsTArray<RefPtr<RefcountedObject>> objArray;
 
   RefcountedObject* a = new RefcountedObject();
   a->AddRef();
@@ -933,6 +1161,46 @@ TEST(TArray, test_swap)
     CHECK_ARRAY(b, data1);
     CHECK_EQ_INT(a.Length(), size_t(0));
   }
+
+  // Test fallible SwapElements of nsTArray.
+  {
+    nsTArray<int> a;
+    nsTArray<int> b;
+
+    a.AppendElements(data1, ArrayLength(data1));
+
+    ASSERT_TRUE(a.SwapElements(b, fallible));
+
+    CHECK_ARRAY(b, data1);
+    CHECK_EQ_INT(a.Length(), size_t(0));
+  }
+
+  // Test fallible SwapElements of FallibleTArray.
+  {
+    FallibleTArray<int> a;
+    FallibleTArray<int> b;
+
+    ASSERT_TRUE(a.AppendElements(data1, ArrayLength(data1), fallible));
+
+    ASSERT_TRUE(a.SwapElements(b, fallible));
+
+    CHECK_ARRAY(b, data1);
+    CHECK_EQ_INT(a.Length(), size_t(0));
+  }
+
+  // Test fallible SwapElements of FallibleTArray with large AutoTArray.
+  {
+    FallibleTArray<int> a;
+    AutoTArray<int, 8192> b;
+
+    ASSERT_TRUE(a.AppendElements(data1, ArrayLength(data1), fallible));
+
+    ASSERT_TRUE(a.SwapElements(b, fallible));
+
+    CHECK_IS_USING_AUTO(b);
+    CHECK_ARRAY(b, data1);
+    CHECK_EQ_INT(a.Length(), size_t(0));
+  }
 }
 
 // Bug 1171296: Disabled on andoid due to crashes.
@@ -991,9 +1259,6 @@ TEST(TArray, test_conversion_operator)
   FallibleTArray<int> f;
   const FallibleTArray<int> fconst;
 
-  InfallibleTArray<int> i;
-  const InfallibleTArray<int> iconst;
-
   nsTArray<int> t;
   const nsTArray<int> tconst;
   AutoTArray<int, 8> tauto;
@@ -1005,10 +1270,6 @@ TEST(TArray, test_conversion_operator)
     ASSERT_EQ((void*)&z1, (void*)&f);           \
     const type<int>& z2 = fconst;               \
     ASSERT_EQ((void*)&z2, (void*)&fconst);      \
-    const type<int>& z5 = i;                    \
-    ASSERT_EQ((void*)&z5, (void*)&i);           \
-    const type<int>& z6 = iconst;               \
-    ASSERT_EQ((void*)&z6, (void*)&iconst);      \
     const type<int>& z9 = t;                    \
     ASSERT_EQ((void*)&z9, (void*)&t);           \
     const type<int>& z10 = tconst;              \
@@ -1020,7 +1281,6 @@ TEST(TArray, test_conversion_operator)
   } while (0)
 
   CHECK_ARRAY_CAST(FallibleTArray);
-  CHECK_ARRAY_CAST(InfallibleTArray);
   CHECK_ARRAY_CAST(nsTArray);
 
 #undef CHECK_ARRAY_CAST
@@ -1037,8 +1297,6 @@ TEST(TArray, test_SetLengthAndRetainStorage_no_ctor)
   const int N = 1050;
   FallibleTArray<int> f;
 
-  InfallibleTArray<int> i;
-
   nsTArray<int> t;
   AutoTArray<int, N> tauto;
 
@@ -1047,7 +1305,6 @@ TEST(TArray, test_SetLengthAndRetainStorage_no_ctor)
 #define FOR_EACH(pre, post) \
   do {                      \
     pre f post;             \
-    pre i post;             \
     pre t post;             \
     pre tauto post;         \
   } while (0)
@@ -1059,10 +1316,9 @@ TEST(TArray, test_SetLengthAndRetainStorage_no_ctor)
   }
 
   void* initial_Hdrs[] = {
-      static_cast<BufAccessor<FallibleTArray<int> >&>(f).GetHdr(),
-      static_cast<BufAccessor<InfallibleTArray<int> >&>(i).GetHdr(),
-      static_cast<BufAccessor<nsTArray<int> >&>(t).GetHdr(),
-      static_cast<BufAccessor<AutoTArray<int, N> >&>(tauto).GetHdr(), nullptr};
+      static_cast<BufAccessor<FallibleTArray<int>>&>(f).GetHdr(),
+      static_cast<BufAccessor<nsTArray<int>>&>(t).GetHdr(),
+      static_cast<BufAccessor<AutoTArray<int, N>>&>(tauto).GetHdr(), nullptr};
 
   // SetLengthAndRetainStorage(n), should NOT overwrite memory when T hasn't
   // a default constructor.
@@ -1070,7 +1326,6 @@ TEST(TArray, test_SetLengthAndRetainStorage_no_ctor)
   FOR_EACH(;, .SetLengthAndRetainStorage(12));
   for (int n = 0; n < 12; ++n) {
     ASSERT_EQ(f[n], n);
-    ASSERT_EQ(i[n], n);
     ASSERT_EQ(t[n], n);
     ASSERT_EQ(tauto[n], n);
   }
@@ -1078,16 +1333,14 @@ TEST(TArray, test_SetLengthAndRetainStorage_no_ctor)
   FOR_EACH(;, .SetLengthAndRetainStorage(N));
   for (int n = 0; n < N; ++n) {
     ASSERT_EQ(f[n], n);
-    ASSERT_EQ(i[n], n);
     ASSERT_EQ(t[n], n);
     ASSERT_EQ(tauto[n], n);
   }
 
   void* current_Hdrs[] = {
-      static_cast<BufAccessor<FallibleTArray<int> >&>(f).GetHdr(),
-      static_cast<BufAccessor<InfallibleTArray<int> >&>(i).GetHdr(),
-      static_cast<BufAccessor<nsTArray<int> >&>(t).GetHdr(),
-      static_cast<BufAccessor<AutoTArray<int, N> >&>(tauto).GetHdr(), nullptr};
+      static_cast<BufAccessor<FallibleTArray<int>>&>(f).GetHdr(),
+      static_cast<BufAccessor<nsTArray<int>>&>(t).GetHdr(),
+      static_cast<BufAccessor<AutoTArray<int, N>>&>(tauto).GetHdr(), nullptr};
 
   // SetLengthAndRetainStorage(n) should NOT have reallocated the internal
   // memory.
