@@ -8,19 +8,19 @@
 #define nsThread_h__
 
 #include "mozilla/Mutex.h"
-#include "nsIIdlePeriod.h"
 #include "nsIThreadInternal.h"
 #include "nsISupportsPriority.h"
 #include "nsThreadUtils.h"
 #include "nsString.h"
 #include "nsTObserverArray.h"
+#include "mozilla/Atomics.h"
 #include "mozilla/Attributes.h"
+#include "mozilla/IntegerTypeTraits.h"
 #include "mozilla/LinkedList.h"
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/SynchronizedEventQueue.h"
 #include "mozilla/NotNull.h"
 #include "mozilla/TimeStamp.h"
-#include "nsAutoPtr.h"
 #include "mozilla/AlreadyAddRefed.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/Array.h"
@@ -42,6 +42,107 @@ class nsThreadEnumerator;
 // See https://www.w3.org/TR/longtasks
 #define LONGTASK_BUSY_WINDOW_MS 50
 
+// A class for managing performance counter state.
+namespace mozilla {
+class PerformanceCounterState {
+ public:
+  explicit PerformanceCounterState(const uint32_t& aNestedEventLoopDepthRef,
+                                   bool aIsMainThread)
+      : mNestedEventLoopDepth(aNestedEventLoopDepthRef),
+        mIsMainThread(aIsMainThread),
+        // Does it really make sense to initialize these to "now" when we
+        // haven't run any tasks?
+        mLastLongTaskEnd(TimeStamp::Now()),
+        mLastLongNonIdleTaskEnd(mLastLongTaskEnd) {}
+
+  class Snapshot {
+   public:
+    Snapshot(uint32_t aOldEventLoopDepth, PerformanceCounter* aCounter,
+             bool aOldIsIdleRunnable)
+        : mOldEventLoopDepth(aOldEventLoopDepth),
+          mOldPerformanceCounter(aCounter),
+          mOldIsIdleRunnable(aOldIsIdleRunnable) {}
+
+    Snapshot(const Snapshot&) = default;
+    Snapshot(Snapshot&&) = default;
+
+   private:
+    friend class PerformanceCounterState;
+
+    const uint32_t mOldEventLoopDepth;
+    // Non-const so we can move out of it and avoid the extra refcounting.
+    RefPtr<PerformanceCounter> mOldPerformanceCounter;
+    const bool mOldIsIdleRunnable;
+  };
+
+  // Notification that a runnable is about to run.  This captures a snapshot of
+  // our current state before we reset to prepare for the new runnable.  This
+  // muast be called after mNestedEventLoopDepth has been incremented for the
+  // runnable execution.  The performance counter passed in should be the one
+  // for the relevant runnable and may be null.  aIsIdleRunnable should be true
+  // if and only if the runnable has idle priority.
+  Snapshot RunnableWillRun(PerformanceCounter* Counter, TimeStamp aNow,
+                           bool aIsIdleRunnable);
+
+  // Notification that a runnable finished executing.  This must be passed the
+  // snapshot that RunnableWillRun returned for the same runnable.  This must be
+  // called before mNestedEventLoopDepth is decremented after the runnable's
+  // execution.
+  void RunnableDidRun(Snapshot&& aSnapshot);
+
+  const TimeStamp& LastLongTaskEnd() const { return mLastLongTaskEnd; }
+  const TimeStamp& LastLongNonIdleTaskEnd() const {
+    return mLastLongNonIdleTaskEnd;
+  }
+
+ private:
+  // Called to report accumulated time, as needed, when we're about to run a
+  // runnable or just finished running one.
+  void MaybeReportAccumulatedTime(TimeStamp aNow);
+
+  // Whether the runnable we are about to run, or just ran, is a nested
+  // runnable, in the sense that there is some other runnable up the stack
+  // spinning the event loop.  This must be called before we change our
+  // mCurrentEventLoopDepth (when about to run a new event) or after we restore
+  // it (after we ran one).
+  bool IsNestedRunnable() const {
+    return mNestedEventLoopDepth > mCurrentEventLoopDepth;
+  }
+
+  // The event loop depth of the currently running runnable.  Set to the max
+  // value of a uint32_t when there is no runnable running, so when starting to
+  // run a toplevel (not nested) runnable IsNestedRunnable() will test false.
+  uint32_t mCurrentEventLoopDepth = std::numeric_limits<uint32_t>::max();
+
+  // A reference to the nsThread's mNestedEventLoopDepth, so we can
+  // see what it is right now.
+  const uint32_t& mNestedEventLoopDepth;
+
+  // A boolean that indicates whether the currently running runnable is an idle
+  // runnable.  Only has a useful value between RunnableWillRun() being called
+  // and RunnableDidRun() returning.
+  bool mCurrentRunnableIsIdleRunnable = false;
+
+  // Whether we're attached to the mainthread nsThread.
+  const bool mIsMainThread;
+
+  // The timestamp from which time to be accounted for should be measured.  This
+  // can be the start of a runnable running or the end of a nested runnable
+  // running.
+  TimeStamp mCurrentTimeSliceStart;
+
+  // Information about when long tasks last ended.
+  TimeStamp mLastLongTaskEnd;
+  TimeStamp mLastLongNonIdleTaskEnd;
+
+  // The performance counter to use for accumulating the runtime of
+  // the currently running event.  May be null, in which case the
+  // event's running time should not be accounted to any performance
+  // counters.
+  RefPtr<PerformanceCounter> mCurrentPerformanceCounter;
+};
+}  // namespace mozilla
+
 // A native thread
 class nsThread : public nsIThreadInternal,
                  public nsISupportsPriority,
@@ -78,7 +179,7 @@ class nsThread : public nsIThreadInternal,
 
  public:
   // The PRThread corresponding to this thread.
-  PRThread* GetPRThread() { return mThread; }
+  PRThread* GetPRThread() const { return mThread; }
 
   const void* StackBase() const { return mStackBase; }
   size_t StackSize() const { return mStackSize; }
@@ -89,6 +190,12 @@ class nsThread : public nsIThreadInternal,
   // nsIThreadManager::NewThread.
   bool ShutdownRequired() { return mShutdownRequired; }
 
+  // Lets GetRunningEventDelay() determine if the pool this is part
+  // of has an unstarted thread
+  void SetPoolThreadFreePtr(mozilla::Atomic<bool, mozilla::Relaxed>* aPtr) {
+    mIsAPoolThreadFree = aPtr;
+  }
+
   void SetScriptObserver(mozilla::CycleCollectedJSContext* aScriptObserver);
 
   uint32_t RecursionDepth() const;
@@ -116,16 +223,12 @@ class nsThread : public nsIThreadInternal,
     EventQueue()->ResumeInputEventPrioritization();
   }
 
-#ifndef RELEASE_OR_BETA
-  mozilla::TimeStamp& NextIdleDeadlineRef() { return mNextIdleDeadline; }
-#endif
-
   mozilla::SynchronizedEventQueue* EventQueue() { return mEvents.get(); }
 
-  bool ShuttingDown() { return mShutdownContext != nullptr; }
+  bool ShuttingDown() const { return mShutdownContext != nullptr; }
 
   virtual mozilla::PerformanceCounter* GetPerformanceCounter(
-      nsIRunnable* aEvent);
+      nsIRunnable* aEvent) const;
 
   size_t SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) const;
 
@@ -139,11 +242,6 @@ class nsThread : public nsIThreadInternal,
 
   static uint32_t MaxActiveThreads();
 
-  const mozilla::TimeStamp& LastLongTaskEnd() { return mLastLongTaskEnd; }
-  const mozilla::TimeStamp& LastLongNonIdleTaskEnd() {
-    return mLastLongNonIdleTaskEnd;
-  }
-
   // When entering local execution mode a new event queue is created and used as
   // an event source. This queue is only accessible through an
   // nsLocalExecutionGuard constructed from the nsLocalExecutionRecord returned
@@ -159,8 +257,13 @@ class nsThread : public nsIThreadInternal,
   // notifications, etc.
   nsLocalExecutionRecord EnterLocalExecution();
 
+  void SetUseHangMonitor(bool aValue) {
+    MOZ_ASSERT(IsOnCurrentThread());
+    mUseHangMonitor = aValue;
+  }
+
  private:
-  void DoMainThreadSpecificProcessing(bool aReallyWait);
+  void DoMainThreadSpecificProcessing() const;
 
  protected:
   friend class nsThreadShutdownEvent;
@@ -205,50 +308,79 @@ class nsThread : public nsIThreadInternal,
   RefPtr<mozilla::ThreadEventTarget> mEventTarget;
 
   // The shutdown contexts for any other threads we've asked to shut down.
-  using ShutdownContexts = nsTArray<nsAutoPtr<struct nsThreadShutdownContext>>;
+  using ShutdownContexts =
+      nsTArray<mozilla::UniquePtr<struct nsThreadShutdownContext>>;
+
+  // Helper for finding a ShutdownContext in the contexts array.
+  struct ShutdownContextsComp {
+    bool Equals(const ShutdownContexts::elem_type& a,
+                const ShutdownContexts::elem_type::Pointer b) const;
+  };
+
   ShutdownContexts mRequestedShutdownContexts;
   // The shutdown context for ourselves.
   struct nsThreadShutdownContext* mShutdownContext;
 
   mozilla::CycleCollectedJSContext* mScriptObserver;
 
-  PRThread* mThread;
   void* mStackBase = nullptr;
   uint32_t mStackSize;
   uint32_t mThreadId;
 
   uint32_t mNestedEventLoopDepth;
-  uint32_t mCurrentEventLoopDepth;
-
-  mozilla::TimeStamp mLastLongTaskEnd;
-  mozilla::TimeStamp mLastLongNonIdleTaskEnd;
 
   mozilla::Atomic<bool> mShutdownRequired;
 
   int8_t mPriority;
 
-  uint8_t mIsMainThread;
-
-  bool IsMainThread() const {
-    return MainThreadFlag(mIsMainThread) == MAIN_THREAD;
-  }
+  const bool mIsMainThread;
+  bool mUseHangMonitor;
+  mozilla::Atomic<bool, mozilla::Relaxed>* mIsAPoolThreadFree;
 
   // Set to true if this thread creates a JSRuntime.
   bool mCanInvokeJS;
 
   bool mHasTLSEntry = false;
 
-  // Used to track which event is being executed by ProcessNextEvent
-  nsCOMPtr<nsIRunnable> mCurrentEvent;
+  // The time the currently running event spent in event queues, and
+  // when it started running.  If no event is running, they are
+  // TimeDuration() & TimeStamp().
+  mozilla::TimeDuration mLastEventDelay;
+  mozilla::TimeStamp mLastEventStart;
 
-  mozilla::TimeStamp mCurrentEventStart;
-  mozilla::TimeStamp mNextIdleDeadline;
+#ifdef EARLY_BETA_OR_EARLIER
+  nsCString mNameForWakeupTelemetry;
+  mozilla::TimeStamp mLastWakeupCheckTime;
+  uint32_t mWakeupCount = 0;
+#endif
 
-  RefPtr<mozilla::PerformanceCounter> mCurrentPerformanceCounter;
+  mozilla::PerformanceCounterState mPerformanceCounterState;
 
   bool mIsInLocalExecutionMode = false;
 };
 
+struct nsThreadShutdownContext {
+  nsThreadShutdownContext(NotNull<nsThread*> aTerminatingThread,
+                          NotNull<nsThread*> aJoiningThread,
+                          bool aAwaitingShutdownAck)
+      : mTerminatingThread(aTerminatingThread),
+        mTerminatingPRThread(aTerminatingThread->GetPRThread()),
+        mJoiningThread(aJoiningThread),
+        mAwaitingShutdownAck(aAwaitingShutdownAck),
+        mIsMainThreadJoining(NS_IsMainThread()) {
+    MOZ_COUNT_CTOR(nsThreadShutdownContext);
+  }
+  MOZ_COUNTED_DTOR(nsThreadShutdownContext)
+
+  // NB: This will be the last reference.
+  NotNull<RefPtr<nsThread>> mTerminatingThread;
+  PRThread* const mTerminatingPRThread;
+  NotNull<nsThread*> MOZ_UNSAFE_REF(
+      "Thread manager is holding reference to joining thread") mJoiningThread;
+  bool mAwaitingShutdownAck;
+  bool mIsMainThreadJoining;
+};
+
 class nsLocalExecutionRecord;
 
 // This RAII class controls the duration of the associated nsThread's local
@@ -295,13 +427,13 @@ class MOZ_TEMPORARY_CLASS nsLocalExecutionRecord final {
 
 class MOZ_STACK_CLASS nsThreadEnumerator final {
  public:
-  nsThreadEnumerator() : mMal(nsThread::ThreadListMutex()) {}
+  nsThreadEnumerator() = default;
 
   auto begin() { return nsThread::ThreadList().begin(); }
   auto end() { return nsThread::ThreadList().end(); }
 
  private:
-  mozilla::OffTheBooksMutexAutoLock mMal;
+  mozilla::OffTheBooksMutexAutoLock mMal{nsThread::ThreadListMutex()};
 };
 
 #if defined(XP_UNIX) && !defined(ANDROID) && !defined(DEBUG) && HAVE_UALARM && \
